"""
Implements the base class for the tables.
"""
from abc import ABCMeta, abstractmethod
import os
import docutils.core
import pandas as pd
import numpy as np

from fitbenchmarking.utils.misc import get_js

FORMAT_DESCRIPTION = \
    {'abs': 'Absolute values are displayed in the table.',
     'rel': 'Relative values are displayed in the table.',
     'both': 'Absolute and relative values are displayed in '
             'the table in the format ``abs (rel)``'}

# The use of Pandas maps means that certain functions do not require use
# of self
# pylint: disable=no-self-use


class Table:
    """
    Base class for the FitBenchmarking HTML and text output tables.
    """
    __metaclass__ = ABCMeta

    def __init__(self, results, best_results, options, group_dir,
                 pp_locations, table_name):
        """
        Initialise the class.

        :param results: results nested array of objects
        :type results: list of list of
                       fitbenchmarking.utils.fitbm_result.FittingResult
        :param best_results: best result for each problem
        :type best_results: list of
                        fitbenchmarking.utils.fitbm_result.FittingResult
        :param options: Options used in fitting
        :type options: utils.options.Options
        :param group_dir: path to the directory where group results should be
                          stored
        :type group_dir: str
        :param pp_locations: tuple containing the locations of the
                             performance profiles (acc then runtime)
        :type pp_locations: tuple(str,str)
        :param table_name: Name of the table
        :type table_name: str
        """
        self.results = results
        self.options = options
        self.best_results = best_results
        self.group_dir = group_dir
        self.pp_locations = pp_locations
        self.table_name = table_name

        colour_scale = self.options.colour_scale

        self.colour_bounds = [colour[0] for colour in colour_scale]
        self.html_colours = [colour[1] for colour in colour_scale]

        self.output_string_type = {"abs": '{:.4g}',
                                   "rel": '{:.4g}',
                                   "both": '{0:.4g} ({1:.4g})'}
        self.has_pp = False
        self.pp_location = ''
        self._table_title = None
        self._file_path = None

    def create_results_dict(self):
        """
        Generates a dictionary used to create HTML and txt tables.

        :return: dictionary containing results where the keys
                 are the problem sets and the values are lists
                 of results objects
        :rtype: dictionary
        """
        results_dict = {}
        for prob_results in self.results:
            name = prob_results[0].name
            results_dict[name] = prob_results
        return results_dict

    @abstractmethod
    def get_values(self, results_dict):
        """
        Gets the main values to be reported in the tables

        :param results_dict: dictionary containing results where the keys
                             are the problem sets and the values are lists
                             of results objects
        :type results_dict: dictionary

        :return: tuple of dictionaries which contain the main values in the
                 tables
        :rtype: tuple
        """
        raise NotImplementedError

    def display_str(self, results):
        """
        Function which converts the results from
        :meth:`~fitbenchmarking.results_processing.base_table.Table.get_values()`
        into a string respresentation to be used in the tables.
        Base class implementation takes
        the absolute and relative values and uses ``self.output_string_type``
        as a template for the string format. This can be overwritten to
        adequately display the results.

        :param results: tuple containing absolute and relative values
        :type results: tuple

        :return: dictionary containing the string representation of the values
                 in the table.
        :rtype: dict
        """
        abs_results, rel_results = results
        comp_mode = self.options.comparison_mode
        result_template = self.output_string_type[self.options.comparison_mode]
        table_output = {}
        for key in abs_results.keys():
            if comp_mode == "abs":
                table_output[key] = [result_template.format(v)
                                     for v in abs_results[key]]
            elif comp_mode == "rel":
                table_output[key] = [result_template.format(v)
                                     for v in rel_results[key]]
            elif comp_mode == "both":
                table_output[key] = [result_template.format(v1, v2)
                                     for v1, v2 in zip(abs_results[key],
                                                       rel_results[key])]
        return table_output

    def get_colour(self, results):
        """
        Converts the result from
        :meth:`~fitbenchmarking.results_processing.base_table.Table.get_values()`
        into the HTML colours
        used in the tables. The base class implementation, for example,
        uses the relative results and ``colour_scale`` within
        :class:`~fitbenchmarking.utils.options.Options`.

        :param results: tuple containing absolute and relative values
        :type results: tuple

        :return: dictionary containing HTML colours for the table
        :rtype: dict
        """
        _, rel_value = results
        colour = {}
        for key, value in rel_value.items():
            if not all(isinstance(elem, list) for elem in value):
                colour_index = np.searchsorted(self.colour_bounds, value)
                colour[key] = [self.html_colours[i]
                               for i in colour_index]
            else:
                colour[key] = []
                for v in value:
                    colour_index = np.searchsorted(self.colour_bounds, v)
                    colour[key].append([self.html_colours[i]
                                        for i in colour_index])
        return colour

    def get_links(self, results_dict):
        """
        Pulls out links to the individual support pages from the results
        object

        :param results: tuple containing absolute and relative values
        :type results: tuple

        :return: dictionary containing links to the support pages
        :rtype: dict
        """
        links = {key: [v.support_page_link for v in value]
                 for key, value in results_dict.items()}
        return links

    def get_error(self, results_dict):
        """
        Pulls out error code from the results object

        :param results: tuple containing absolute and relative values
        :type results: tuple

        :return: dictionary containing error codes from the minimizers
        :rtype: dict
        """
        error = {key: [v.error_flag for v in value]
                 for key, value in results_dict.items()}
        return error

    def create_pandas_data_frame(self, str_results):
        """
        Converts dictionary of results into a pandas data frame

        :param str_results: dictionary containing the string representation of
                            the values in the table.
        :type str_results: dict

        :return: pandas data frame with from results
        :rtype: Pandas DataFrame
        """
        table = pd.DataFrame.from_dict(str_results, orient='index')
        minimizers_list = [(s, m) for s in self.options.software
                           for m in self.options.minimizers[s]]
        columns = pd.MultiIndex.from_tuples(minimizers_list)
        table.columns = columns
        return table

    def to_html(self, table, colour, links, error):
        """
        Takes Pandas data frame and converts it into the HTML table output

        :param table: pandas data frame with from results
        :type table: Pandas DataFrame
        :param colour: dictionary containing error codes from the minimizers
        :type colour: dict
        :param links: dictionary containing links to the support pages
        :type links: dict
        :param error: dictionary containing error codes from the minimizers
        :type error: dict

        :return: HTLM table output
        :rtype: str
        """
        link_template = '<a href="https://fitbenchmarking.readthedocs.io/'\
                        'en/latest/users/options/minimizer_option.html#'\
                        '{0}-{0}" target="_blank">{0}</a>'
        table.apply(lambda x: self.enable_error(x, error, "<sup>{}</sup>"),
                    axis=1, result_type='expand')
        table.apply(lambda x: self.enable_link(x, links), axis=1,
                    result_type='expand')

        minimizers_list = [(link_template.format(s.replace("_", "-")), m)
                           for s in self.options.software
                           for m in self.options.minimizers[s]]
        columns = pd.MultiIndex.from_tuples(minimizers_list)
        table.columns = columns

        index = []
        for b, i in zip(self.best_results, table.index):
            rel_path = os.path.relpath(path=b.support_page_link,
                                       start=self.group_dir)
            index.append('<a href="{0}">{1}</a>'.format(rel_path, i))
        table.index = index
        table_style = table.style.apply(
            lambda x: self.colour_highlight(x, colour), axis=1)

        return table_style.render()

    def to_txt(self, table, error):
        """
        Takes Pandas data frame and converts it into the plain text table
        output

        :param table: pandas data frame with from results
        :type table: Pandas DataFrame
        :param error: dictionary containing error codes from the minimizers
        :type error: dict

        :return: plain text table output
        :rtype: str
        """
        table.apply(lambda x: self.enable_error(x, error, "[{}]"),
                    axis=1, result_type='expand')
        return table.to_string()

    def enable_link(self, value, links):
        """
        Enable HTML links in values

        :param value: Row data from the pandas array
        :type value: pandas.core.series.Series
        :param links: dictionary containing links to the support pages
        :type links: dict

        :return: Row data from the pandas array with links enabled
        :rtype: pandas.core.series.Series
        """
        name = value.name
        support_page_link = links[name]
        i = 0
        for l, v in zip(support_page_link, value.array):
            tmp_link = os.path.relpath(path=l,
                                       start=self.group_dir)
            value.array[i] = '<a href="{0}">{1}</a>'.format(tmp_link, v)
            i += 1
        return value

    def enable_error(self, value, error, template):
        """
        Enable error codes in table

        :param value: Row data from the pandas array
        :type value: pandas.core.series.Series
        :param error: dictionary containing error codes from the minimizers
        :type error: dict

        :return: Row data from the pandas array with error codes enabled
        :rtype: pandas.core.series.Series
        """
        name = value.name
        error_code = [template.format(e) if e != 0 else ""
                      for e in error[name]]
        i = 0
        for v, e in zip(value, error_code):
            value.array[i] = v + e
            i += 1
        return value

    def colour_highlight(self, value, colour):
        """
        Takes the HTML colour values from
        :meth:`~fitbenchmarking.results_processing.base_table.Table.get_colour`
        and maps it over the HTML table using the Pandas style mapper.

        :param value: Row data from the pandas array
        :type value: pandas.core.series.Series
        :param colour: dictionary containing error codes from the minimizers
        :type colour: dict

        :return: list of HTML colours
        :rtype: list
        """
        color_template = 'background-color: {0}'
        name = value.name.split('"')[2].replace("</a>", "")[1:]
        colour_style = colour[name]
        output_colour = []
        for c in colour_style:
            output_colour.append(color_template.format(c))
        return output_colour

    def get_description(self, html_description):
        """
        Generates table description from class docstrings and converts them
        into html

        :param html_description: Dictionary containing table descriptions
        :type html_description: dict

        :return: Dictionary containing table descriptions
        :rtype: dict
        """
        FORMAT_DESCRIPTION[self.name] = self.__doc__
        for name in [self.name, self.options.comparison_mode]:
            descrip = FORMAT_DESCRIPTION[name]
            descrip = descrip.replace(':ref:', '')
            js = get_js(self.options, self.group_dir)
            docsettings = {
                'math_output': 'MathJax '+js['mathjax']
            }
            description_page = docutils.core.publish_parts(
                descrip,
                writer_name='html',
                settings_overrides=docsettings)
            html_description[name] = description_page['body']
            html_description[name] = html_description[name].replace(
                '<blockquote>\n', '')
        return html_description

    @property
    def table_title(self):
        """
        Getter function for table name if self._table_title is None

        :return: name of table
        :rtype: str
        """
        if self._table_title is None:
            self._table_title = "FitBenchmarking: {0} table".format(self.name)
        return self._table_title

    @table_title.setter
    def table_title(self, value):
        """
        Setting function to set the name of the table

        :param value: name of table
        :type value: str
        """
        self._table_title = value

    @property
    def file_path(self):
        """
        Getter function for the path to the table

        :return: path to table
        :rtype: str
        """
        if self._file_path is None:
            self._file_path = os.path.join(self.group_dir, self.table_name)
        return self._file_path

    @file_path.setter
    def file_path(self, value):
        """
        Setting function to set the path to the table

        :param value: path to table
        :type value: str
        """
        self._file_path = value



"""
Module which calculates SciPy 3 point finite difference approximation
"""
import numpy as np
from scipy.optimize._numdiff import approx_derivative

from fitbenchmarking.jacobian.base_jacobian import Jacobian

# pylint: disable=useless-super-delegation
class ScipyThreePoint(Jacobian):
    """
    Implements SciPy 3 point finite difference approximation to the derivative
    """

    def __init__(self, problem):
        super(ScipyThreePoint, self).__init__(problem)

    def eval(self, params, func=None, **kwargs):
        """
        Evaluates Jacobian

        :param params: The parameter values to find the Jacobian at
        :type params: list
        :param func: Function to find the Jacobian for, defaults to
                     problem.eval_r
        :type func: Callable, optional

        :return: Approximation of the Jacobian
        :rtype: numpy array
        """
        if func is None:
            func = self.problem.eval_r

        jac = approx_derivative(func, params, method="3-point",
                                rel_step=None, f0=func(params, **kwargs),
                                bounds=(-np.inf, np.inf),
                                kwargs=kwargs)
        return jac



"""
Tests for fitbenchmarking.core.fitting_benchmarking.loop_over_minimizers
"""
from __future__ import (absolute_import, division, print_function)
import inspect
import os
import unittest

from fitbenchmarking import mock_problems
from fitbenchmarking.utils import fitbm_result, output_grabber
from fitbenchmarking.core.fitting_benchmarking import loop_over_minimizers
from fitbenchmarking.parsing.parser_factory import parse_problem_file
from fitbenchmarking.controllers.base_controller import Controller
from fitbenchmarking.utils.options import Options
from fitbenchmarking.jacobian.SciPyFD_2point_jacobian import ScipyTwoPoint


# Due to construction of the controllers two folder functions
# pylint: disable=unnecessary-pass
class DummyController(Controller):
    """
    Minimal instantiatable subclass of Controller class for testing
    """

    def __init__(self, problem):
        """
        Initialize dummy controller

        :param problem: Problem to fit
        :type problem: FittingProblem
        """
        super(DummyController, self).__init__(problem)
        self.algorithm_check = {'all': ['deriv_free_algorithm', 'general'],
                                'ls': [None],
                                'deriv_free': ['deriv_free_algorithm'],
                                'general': ['general']}
        self.final_params_expected = [[1, 2, 3, 4], [4, 3, 2, 1]]
        self.flag_expected = [0, 1]
        self.count = 0

    def setup(self):
        """
        Mock controller setup function
        """
        pass

    def fit(self):
        """
        Mock controller fit function
        """
        pass

    def cleanup(self):
        """
        Mock controller cleanup function
        """
        self.final_params = self.final_params_expected[self.count]
        self.flag = self.flag_expected[self.count]
        self.count += 1


def make_fitting_problem(file_name='cubic.dat', minimizers=None):
    """
    Helper function that returns a simple fitting problem
    """
    options = Options()
    if minimizers:
        options.minimizers = minimizers

    bench_prob_dir = os.path.dirname(inspect.getfile(mock_problems))
    fname = os.path.join(bench_prob_dir, file_name)

    fitting_problem = parse_problem_file(fname, options)
    fitting_problem.correct_data()
    jac = ScipyTwoPoint(fitting_problem)
    fitting_problem.jac = jac
    return fitting_problem


class LoopOverMinimizersTests(unittest.TestCase):
    """
    loop_over_minimizers tests
    """

    def setUp(self):
        """
        Setting up problem for tests
        """
        self.minimizers = ["deriv_free_algorithm", "general"]
        self.problem = make_fitting_problem(minimizers=self.minimizers)
        self.controller = DummyController(problem=self.problem)
        self.options = self.problem.options
        self.grabbed_output = output_grabber.OutputGrabber(self.options)
        self.controller.parameter_set = 0

    def shared_tests(self):

        """
        Shared tests for the `loop_over_minimizer` function
        """

        results_problem, minimizer_failed = loop_over_minimizers(
            self.controller, self.minimizers, self.options,
            self.grabbed_output)
        algorithms = \
            self.controller.algorithm_check[self.options.algorithm_type]
        unselected_algorithms = [x for x in self.minimizers
                                 if x not in algorithms]
        assert len(results_problem) == len(algorithms)
        assert all(isinstance(x, fitbm_result.FittingResult)
                   for x in results_problem)
        for i, result in enumerate(results_problem):
            assert result.params == self.controller.final_params_expected[i]
            assert result.error_flag == self.controller.flag_expected[i]
        assert minimizer_failed == unselected_algorithms

    def test_run_all_minimzers(self):
        """
        Tests that all minimizers run
        """
        self.shared_tests()

    def test_run_minimzers_deriv_free(self):
        """
        Tests that ``algorithm_type = deriv_free`` minimizers run
        """
        self.options.algorithm_type = "deriv_free"
        self.shared_tests()


if __name__ == "__main__":
    unittest.main()



"""
Implements the base Parser as a Context Manager.
"""

from abc import ABCMeta, abstractmethod


class Parser:
    """
    Base abstract class for a parser.
    Further parsers should inherit from this and override the abstract parse()
    method.
    """
    __metaclass__ = ABCMeta

    def __init__(self, filename, options):
        """
        Store the filename for use by enter.

        :param filename: The path to the file to be parsed
        :type filename: string
        :param options: all the information specified by the user
        :type options: fitbenchmarking.utils.options.Options
        """
        self._filename = filename
        self.options = options
        self.file = None
        self.fitting_problem = None

    def __enter__(self):
        """
        Called when used as a context manager.
        Opens the file ready for parsing.
        """
        self.file = open(self._filename, 'r')
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Called when used as a context manager.
        Closes the file.

        :param exc_type: Used if an exception occurs. Contains the
                         exception type.
        :type exc_type: type
        :param exc_value: Used if an exception occurs. Contains the exception
                      value.
        :type exc_value: Exception
        :param traceback: Used if an exception occurs. Contains the exception
                          traceback.
        :type traceback: traceback
        """
        try:
            self.file.close()
        except AttributeError:
            pass

    @abstractmethod
    def parse(self):
        """
        Parse the file into a FittingProblem.

        :return: The parsed problem
        :rtype: FittingProblem
        """
        raise NotImplementedError



"""
Tests for the exception handler file.
"""
from unittest import TestCase

from fitbenchmarking.cli.exception_handler import exception_handler
from fitbenchmarking.utils.exceptions import FittingProblemError


class TestExceptionHandler(TestCase):
    """
    Tests for the exception handler function in exception handler.
    """
    def test_return_unchanged(self):
        """
        Test that the return value is correct when wrapping a function.
        """
        @exception_handler
        def get_val():
            """
            Do something...
            """
            return "The correct string!"

        result = get_val()
        self.assertEqual(result, "The correct string!")

    def test_non_fb_exception(self):
        """
        Test behaviour on a non fitbenchmarking exception.
        """
        @exception_handler
        def get_val():
            """
            Do something...
            """
            raise RuntimeError

        self.assertRaises(SystemExit, get_val)

    def test_fb_exception(self):
        """
        Test behaviour on a fitbenchmarking exception.
        """
        @exception_handler
        def get_val():
            """
            Do something...
            """
            raise FittingProblemError

        self.assertRaises(SystemExit, get_val)



import os
import sys
import platform

from fitbenchmarking.utils.log import get_logger

LOGGER = get_logger()


class OutputGrabber:
    """
    Class used to grab outputs for stdout and stderr
    """
    def __init__(self, options):
        self.stdout_grabber = StreamGrabber(sys.stdout)
        self.stderr_grabber = StreamGrabber(sys.stderr)

        # From issue 500 the output grabber does not currently on windows, thus
        # we set the __enter__ and __exit__ functions to pass through for this
        # case
        self.system = platform.system() != "Windows"

        self.external_output = options.external_output

    def __enter__(self):
        if self.system and self.external_output != 'debug':
            self.stdout_grabber.start()
            self.stderr_grabber.start()
        return self

    def __exit__(self, type, value, traceback):
        if self.system and self.external_output:
            self.stdout_grabber.stop()
            self.stderr_grabber.stop()
            if self.stdout_grabber.capturedtext:
                self._log('Captured output: \n%s',
                          self.stdout_grabber.capturedtext)

            if self.stderr_grabber.capturedtext:
                self._log('Captured error: \n%s',
                          self.stderr_grabber.capturedtext)

    def _log(self, str_log, *args):
        if self.external_output == 'log_only':
            LOGGER.debug(str_log, *args)
        else:
            LOGGER.info(str_log, *args)


class StreamGrabber:
    """
    Class used to grab an output stream.
    """
    escape_char = "\b"

    def __init__(self, stream):

        self.origstream = stream
        self.origstreamfd = stream.fileno()
        self.encoding = stream.encoding

        self.capturedtext = ""

    def start(self):
        """
        Start capturing the stream data.
        """
        # Create a pipe so the stream can be captured:
        self.pipe_out, self.pipe_in = os.pipe()
        self.capturedtext = ""

        # Save a copy of the stream:
        self.streamfd = os.dup(self.origstreamfd)

        # Replace the original stream with our write pipe:
        os.dup2(self.pipe_in, self.origstreamfd)

    def stop(self):
        """
        Stop capturing the stream data and save the text in `capturedtext`.
        """
        # Print the escape character to make the readOutput method stop:
        self.origstream.write(self.escape_char)
        # Flush the stream to make sure all our data goes in before
        # the escape character:
        self.origstream.flush()

        # Reads the output and stores it in capturedtext
        self.readOutput()

        # Close the pipes:
        os.close(self.pipe_in)
        os.close(self.pipe_out)

        # Restore the original streams:
        os.dup2(self.streamfd, self.origstreamfd)

        # Close the duplicate streams:
        os.close(self.streamfd)

    def readOutput(self):
        """
        Read the stream data (one byte at a time)
        and save the text in `capturedtext`.
        """
        while True:
            char = os.read(self.pipe_out, 1).decode(self.encoding)
            if not char or self.escape_char in char:
                break
            self.capturedtext += char



from __future__ import (absolute_import, division, print_function)
import inspect
import os
import shutil
import time
import unittest

from fitbenchmarking import mock_problems
from fitbenchmarking.utils.misc import get_problem_files
from fitbenchmarking.utils.misc import get_css, get_js
from fitbenchmarking.utils.options import Options

class CreateDirsTests(unittest.TestCase):

    def base_path(self):
        """
        Helper function that returns the path to
        /fitbenchmarking/benchmark_problems
        """
        bench_prob_dir = os.path.dirname(inspect.getfile(mock_problems))
        return bench_prob_dir

    def setUp(self):
        """
        Create some datafiles to look for.
        """
        self.dirname = os.path.join(self.base_path(),
                                    'mock_datasets_{}'.format(time.time()))
        os.mkdir(self.dirname)

        expected = []
        for i in range(10):
            filename = 'file_{}.txt'.format(i)
            filepath = os.path.join(self.dirname, filename)
            expected.append(filepath)

            with open(filepath, 'w+') as f:
                f.write('This is a mock data file to check that finding files'
                        'is correct')

        self.expected = sorted(expected)

    def tearDown(self):
        """
        Clean up created datafiles.
        """
        shutil.rmtree(self.dirname)

    def test_getProblemFiles_get_correct_probs(self):
        """
        Test that the correct files are found
        """

        problems = get_problem_files(self.dirname)

        self.assertIsInstance(problems, list)
        self.assertEqual(self.expected, sorted(problems))

    def test_get_css(self):
        
        options = Options()
        print(options.results_dir)
        test_dir = os.path.join(options.results_dir,"foo")
        
        expected_css_dir = os.path.join("..","css")
        expected_main_css = os.path.join(expected_css_dir,"main_style.css")
        expected_table_css = os.path.join(expected_css_dir,"table_style.css")
        expected_custom_css = os.path.join(expected_css_dir,"custom_style.css")
        css = get_css(options,test_dir)

        self.assertEqual(css['main'], expected_main_css)
        self.assertEqual(css['table'], expected_table_css)
        self.assertEqual(css['custom'], expected_custom_css)

    def test_get_js(self):
        
        options = Options()
        print(options.results_dir)
        test_dir = os.path.join(options.results_dir,"foo")
        
        expected_js_dir = os.path.join("..","js")
        expected_mathjax_js = os.path.join(expected_js_dir,"tex-mml-chtml.js")
        js = get_js(options,test_dir)

        self.assertEqual(js['mathjax'], expected_mathjax_js)

if __name__ == "__main__":
    unittest.main()



import os

"""
This script was used to re-format data files specified in FitBenchmark problem definition files in the sub-folder
'data_files' of Muon_data and Neutron_data.

If the data error column of the data files is 'nan' or 0 they are changed to 1. The latter is in accordance with the
default fitting behaviour in Mantid. 
"""

file_dir = os.path.dirname(os.path.realpath(__file__))
bp_path = os.path.dirname(file_dir)
muon_data_path = os.path.join(bp_path, 'Muon_data', 'data_files')
neutron_data_path = os.path.join(bp_path, 'Neutron_data', 'data_files')

paths = [muon_data_path, neutron_data_path]

for path in paths:
    for root, dirs, files in os.walk(path):
        for name in files:
            if os.path.splitext(name)[1] == '.txt':
                data_file_path = os.path.join(root, name)

                data_file = open(data_file_path, 'r+')

                data_string = (data_file.readlines())

                for idx, line in enumerate(data_string[2:]):
                    line = line.strip()
                    point_text = line.split()
                    if point_text[2] == 'nan':
                        point_text[2] = '1.000000e+00'
                        sentence = '    '.join(point_text)
                        data_string[idx+2] = sentence + '\n'
                    elif float(point_text[2]) == 0.0:
                        point_text[2] = '1.000000e+00'
                        sentence = '    '.join(point_text)
                        data_string[idx+2] = sentence + '\n'

                data_file.seek(0)
                data_file.writelines(data_string)
                data_file.close()




from collections import OrderedDict
from unittest import TestCase

import numpy as np

from fitbenchmarking.parsing.fitting_problem import FittingProblem
from fitbenchmarking.utils.options import Options
from fitbenchmarking.utils import exceptions
from fitbenchmarking.jacobian.SciPyFD_2point_jacobian import ScipyTwoPoint
from fitbenchmarking.jacobian.SciPyFD_3point_jacobian import ScipyThreePoint
from fitbenchmarking.jacobian.SciPyFD_cs_jacobian import ScipyCS
from fitbenchmarking.jacobian.jacobian_factory import create_jacobian


def f(x, p1, p2):
    """
    Test function for numerical Jacobians

    :param x: x data points, defaults to self.data_x
    :type x: numpy array, optional
    :param p1: parameter 1
    :type p1: float
    :param p2: parameter 1
    :type p2: float

    :return: function evaluation
    :rtype: numpy array
    """
    return p1 * np.exp(p2 * x)


def J(x, p):
    """
    Analytic Jacobian evaluation

    :param x: x data points, defaults to self.data_x
    :type x: numpy array, optional
    :param p: list of parameters to fit
    :type p: list

    :return: Jacobian evaluation
    :rtype: numpy array
    """
    return np.column_stack((-np.exp(p[1] * x),
                            -x * p[0] * np.exp(p[1] * x)))


class TestJacobianClass(TestCase):
    """
    Tests for Jacobian classes
    """

    def setUp(self):
        """
        Setting up tests
        """
        options = Options()
        self.fitting_problem = FittingProblem(options)
        self.fitting_problem.function = f
        self.fitting_problem.data_x = np.array([1, 2, 3, 4, 5])
        self.fitting_problem.data_y = np.array([1, 2, 4, 8, 16])
        self.params = [6, 0.1]
        self.actual = J(x=self.fitting_problem.data_x, p=self.params)

    def test_scipy_two_point_eval(self):
        """
        Test for ScipyTwoPoint evaluation is correct
        """
        jac = ScipyTwoPoint(self.fitting_problem)
        eval_result = jac.eval(params=self.params)
        self.assertTrue(np.isclose(self.actual, eval_result).all())

    def test_scipy_three_point_eval(self):
        """
        Test for ScipyThreePoint evaluation is correct
        """
        jac = ScipyThreePoint(self.fitting_problem)
        eval_result = jac.eval(params=self.params)
        self.assertTrue(np.isclose(self.actual, eval_result).all())

    def test_scipy_cs_eval(self):
        """
        Test for ScipyCS evaluation is correct
        """
        jac = ScipyCS(self.fitting_problem)
        eval_result = jac.eval(params=self.params)
        self.assertTrue(np.isclose(self.actual, eval_result).all())


class FactoryTests(TestCase):
    """
    Tests for the Jacobian factory
    """

    def test_imports(self):
        """
        Test that the factory returns the correct class for inputs
        """
        self.options = Options()

        valid = ['2point', '3point', 'cs']
        expected_name = ["scipytwopoint", "scipythreepoint", "scipycs"]

        invalid = ['foo', 'bar', 'hello', 'r2d2']

        for v, e in zip(valid, expected_name):
            self.options.num_method = v
            jac = create_jacobian(self.options)
            self.assertTrue(jac.__name__.lower().startswith(e))

        for i in invalid:
            self.options.num_method = i
            self.assertRaises(exceptions.NoJacobianError,
                              create_jacobian,
                              self.options)



"""
This file implements a parser for the NIST style data format.
"""

from __future__ import (absolute_import, division, print_function)

from collections import OrderedDict
import re

import numpy as np

from fitbenchmarking.parsing.base_parser import Parser
from fitbenchmarking.parsing.fitting_problem import FittingProblem
from fitbenchmarking.parsing.nist_data_functions import nist_func_definition
from fitbenchmarking.utils.exceptions import ParsingError
from fitbenchmarking.utils.log import get_logger

LOGGER = get_logger()


# By design the parsers may have functions that do not necessarily use self
# within them
# pylint: no-self-use
class NISTParser(Parser):
    """
    Parser for the NIST problem definition file.
    """

    def parse(self):
        """
        Parse the NIST problem file into a Fitting Problem.

        :return: The fully parsed fitting problem
        :rtype: fitbenchmarking.parsing.fitting_problem.FittingProblem
        """

        fitting_problem = FittingProblem(self.options)

        equation, data, starting_values, name = self._parse_line_by_line()
        data = self._parse_data(data)

        fitting_problem.data_x = data[:, 1]
        fitting_problem.data_y = data[:, 0]
        if len(data[0, :]) > 2:
            fitting_problem.data_e = data[:, 2]

        fitting_problem.name = name

        # String containing a mathematical expression
        fitting_problem.equation = self._parse_equation(equation)

        fitting_problem.starting_values = starting_values

        fitting_problem.function = \
            nist_func_definition(function=fitting_problem.equation,
                                 param_names=starting_values[0].keys())
        return fitting_problem

    def _parse_line_by_line(self):
        """
        Parses the NIST file one line at the time.

        :return: the equation, data pattern, and starting values
                 sections of the file
        :rtype: str, str, list of list of float
        """
        lines = self.file.readlines()
        idx, ignored_lines = 0, 0

        while idx < len(lines):
            line = lines[idx].strip()
            idx += 1
            if not line:
                continue

            if line.startswith('Model:'):
                equation_text, idx = self._get_nist_model(lines, idx)
            elif 'Starting values' in line or 'Starting Values' in line:
                starting_values, idx = self._get_nist_starting_values(lines,
                                                                      idx)
            elif line.startswith("Data:"):
                if " x" in line and " y " in line:
                    data_pattern_text, idx = self._get_data_txt(lines, idx)
            elif line.startswith("Dataset Name:"):
                name = line.split(':', 1)[1]
                name = name.split('(', 1)[0]
                name = name.strip()
            else:
                ignored_lines += 1

        LOGGER.debug("%s lines were ignored in this problem file",
                     ignored_lines)

        return equation_text, data_pattern_text, starting_values, name

    def _get_nist_model(self, lines, idx):
        """
        Gets the model equation used in the fitting process from the
        NIST file.

        :param lines: All lines in the imported nist file
        :type lines: list of str
        :param idx: the line at which the parser is at
        :type idx: int

        :return: The equation from the NIST file and the
                 new index
        :rtype: str and int
        """

        equation_text, idxerr = None, False
        try:
            while (not re.match(r'\s*y\s*=(.+)', lines[idx])
                   and not re.match(r'\s*log\[y\]\s*=(.+)', lines[idx]))\
                    and idx < len(lines):

                idx += 1
        except IndexError:
            LOGGER.error("Could not find equation, index went out of bounds!")
            idxerr = True

        equation_text, idx = self._get_equation_text(lines, idxerr, idx)

        return equation_text, idx

    def _get_equation_text(self, lines, idxerr, idx):
        """
        Gets the equation text from the NIST file.

        :param lines: All lines in the imported nist file
        :type lines: list of str
        :param idxerr: boolean that points out if there were any problems
                       in finding the equation in the file
        :type idxerr: bool
        :param idx: the line at which the parser is at
        :type idx: int

        :return: The equation from the NIST file and the
                 new index
        :rtype: str and int
        """

        # Next non-empty lines are assumed to continue the equation
        equation_text = ''
        if idxerr is False:
            while lines[idx].strip():
                equation_text += lines[idx].strip()
                idx += 1

        if not equation_text:
            raise ParsingError("Could not find the equation!")

        return equation_text, idx

    def _get_data_txt(self, lines, idx):
        """
        Gets the data pattern from the NIST problem file.

        :param lines: All lines in the imported nist file
        :type lines: list of str
        :param idx: the line at which the parser is at
        :type idx: int

        :return: The data pattern and the new index
        :rtype: (list of str) and int
        """

        data_text = None
        data_text = lines[idx:]
        idx = len(lines)

        if not data_text:
            raise ParsingError("Could not find the data!")

        return data_text, idx

    def _parse_data(self, data_text):
        """
        Parses the data string and returns a numpy array of the
        data points of the problem.

        :param data_text: The data from the NIST problem file
        :type data_text: list of str

        :return: The data points of the problem
        :rtype: np.ndarray
        """

        if not data_text:
            return None

        first = data_text[0].strip()
        dim = len(first.split())
        data_points = np.zeros((len(data_text), dim))

        for idx, line in enumerate(data_text):
            line = line.strip()
            point_text = line.split()
            point = [float(val) for val in point_text]
            data_points[idx, :] = point

        data_points = self._sort_data_from_x_data(data_points)

        return data_points

    def _sort_data_from_x_data(self, data_points):
        """
        Sort the numpy array of the data points of the problem
        using its x data.

        :param data_points: Unsorted data points of the problem
        :type data_points: np.ndarray

        :return: sorted data points of the problem
        :rtype: np.ndarray
        """

        sorted_data_points = sorted(data_points, key=lambda x: x[1])

        return np.asarray(sorted_data_points)

    def _parse_equation(self, eq_text):
        """
        Parses the equation and converts it to the right format.

        :param eq_text: The equation
        :type eq_text: str

        :return: formatted equation
        :rtype: str
        """

        start_normal = r'\s*y\s*=(.+)'
        if re.match(start_normal, eq_text):
            match = re.search(r'y\s*=(.+)\s*\+\s*e', eq_text)
            equation = match.group(1).strip()
        else:
            raise ParsingError("Unrecognized equation syntax when trying to "
                               "parse a NIST equation: " + eq_text)

        equation = self._convert_nist_to_muparser(equation)
        return equation

    def _convert_nist_to_muparser(self, equation):
        """
        Converts the raw equation from the NIST file into muparser format.

        :param equation: Raw equation
        :type equation: str

        :return: formatted muparser equation (mathematical notation)
        :rtype: str
        """

        # 'NIST equation syntax' => muparser syntax
        equation = equation.replace('[', '(')
        equation = equation.replace(']', ')')
        equation = equation.replace('arctan', 'atan')
        equation = equation.replace('**', '^')
        return equation

    def _get_nist_starting_values(self, lines, idx):
        """
        Gets the function starting values from the NIST problem file.

        :param lines: The lines in the imported nist file
        :type lines: list of str
        :param idx: the line at which the parser is at
        :type idx: int

        :return: The starting values and the new index
        :rtype: (list of OrderedDict) and int
        """

        starting_values = None
        idx += 2
        starting_values = self._parse_starting_values(lines[idx:])
        idx += len(starting_values)

        return starting_values, idx

    def _parse_starting_values(self, lines):
        """
        Parses the starting values of a NIST file and converts them into an
        array.

        :param lines: All lines in the imported nist file
        :type lines: list of str

        :return: The starting values used in NIST problem
        :rtype: list of list of float
        """
        starting_vals = []
        for line in lines:
            if not line.strip() or line.startswith('Residual'):
                break

            startval_str = line.split()
            if not startval_str[0].isalnum():
                raise ParsingError('Could not parse starting parameters.')

            alt_values = self._get_startvals_floats(startval_str)

            if not starting_vals:
                starting_vals = [OrderedDict() for _ in alt_values]
            for i, a in enumerate(alt_values):
                starting_vals[i][startval_str[0]] = a

        return starting_vals

    def _get_startvals_floats(self, startval_str):
        """
        Converts the starting values into floats.

        :param startval_str: Unparsed starting values
        :type startval_str: list of str

        :return: Starting values array of floats
        :rtype: list of float
        """

        # A bit weak/lax parsing, if there is one less column,
        # assume only one start point
        if len(startval_str) == 6:
            alt_values = [float(startval_str[2]), float(startval_str[3])]
        elif len(startval_str) == 5:
            alt_values = [float(startval_str[2])]
        # In the NIST format this can only contain 5 or 6 columns
        else:
            raise ParsingError("Failed to parse this line as starting "
                               "values information: {0}".format(startval_str))

        return alt_values



'''
Test the PLOTTING section for the options file
'''
import inspect
import shutil
import os
import numpy as np
import unittest

from fitbenchmarking.utils import exceptions
from fitbenchmarking.utils.options import Options


class PlottingOptionTests(unittest.TestCase):
    """
    Checks that the defaults in the PLOTTING section are set correctly
    """

    def setUp(self):
        """
        Initializes options class with defaults
        """
        self.options = Options()

    def test_make_plots_default(self):
        """
        Checks make_plots default
        """
        expected = True
        actual = self.options.make_plots
        self.assertEqual(expected, actual)

    def test_colour_scale_default(self):
        """
        Checks colour_scale default
        """
        expected = [(1.1, "#fef0d9"),
                    (1.33, "#fdcc8a"),
                    (1.75, "#fc8d59"),
                    (3, "#e34a33"),
                    (np.inf, "#b30000")]
        actual = self.options.colour_scale
        self.assertEqual(expected, actual)

    def test_comparison_mode_default(self):
        """
        Checks comparison_mode default
        """
        expected = 'both'
        actual = self.options.comparison_mode
        self.assertEqual(expected, actual)

    def test_table_type_default(self):
        """
        Checks table_type default
        """
        expected = ['acc', 'runtime', 'compare', 'local_min']
        actual = self.options.table_type
        self.assertEqual(expected, actual)

    def test_results_dir_default(self):
        """
        Checks results_dir default
        """
        expected = os.path.abspath('fitbenchmarking_results')
        actual = self.options.results_dir
        self.assertEqual(expected, actual)


class UserPlottingOptionTests(unittest.TestCase):
    """
    Checks the plotting in the options file are set correctly or raise errors
    """

    def setUp(self):
        """
        Sets the directory to save the temporary ini files in
        """
        options_dir = os.path.dirname(inspect.getfile(Options))
        self.test_files_dir = os.path.join(options_dir, 'tests', 'files')
        os.mkdir(self.test_files_dir)

    def tearDown(self):
        """
        Deletes temporary folder and results produced
        """
        if os.path.exists(self.test_files_dir):
            shutil.rmtree(self.test_files_dir)

    def generate_user_ini_file(self, opt_name, config_str):
        """
        Generates user defined ini file

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: section of an ini file which sets the option
        :type config_str: str

        :return: location of temporary ini file
        :rtype: str
        """
        opts_file = os.path.join(self.test_files_dir,
                                 'test_{}_valid.ini'.format(opt_name))
        with open(opts_file, 'w') as f:
            f.write(config_str)
        return opts_file

    def shared_valid(self, opt_name, options_set, config_str):
        """
        Shared test to check that the plotting option set is valid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param options_set: option set to be tested
        :type options_set: list
        :param config_str: section of an ini file which sets the option
        :type config_str: str
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        options = Options(opts_file)
        actual = getattr(options, opt_name)
        self.assertEqual(options_set, actual)

    def shared_invalid(self, opt_name, config_str):
        """
        Shared test to check that the plotting option set is invalid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: option set to be tested
        :type config_str: list
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        with self.assertRaises(exceptions.OptionsError):
            Options(opts_file)

    def test_invalid_option_key(self):
        """
        Tests that the user defined option key is invalid.
        """
        config_str = \
            "[PLOTTING]\nmake_figures: True"
        self.shared_invalid('make_figures', config_str)

    def test_minimizer_make_plots_valid(self):
        """
        Checks user set make_plots is valid
        """
        set_option = False
        config_str = \
            "[PLOTTING]\nmake_plots: no"
        self.shared_valid('make_plots', set_option, config_str)

    def test_minimizer_make_plots_invalid(self):
        """
        Checks user set make_plots is invalid
        """
        config_str = \
            "[PLOTTING]\nmake_plots: a selection of plots"
        self.shared_invalid('make_plots', config_str)

    def test_minimizer_colour_scale_valid(self):
        """
        Checks user set colour_scale is valid
        """
        set_option = [(1.1, "#fef0d9"),
                      (1.33, "#fdcc8a")]
        config_str = \
            "[PLOTTING]\ncolour_scale: 1.1, #fef0d9\n 1.33, #fdcc8a"
        self.shared_valid('colour_scale', set_option, config_str)

    def test_minimizer_comparison_mode_valid(self):
        """
        Checks user set comparison_mode is valid
        """
        set_option = "abs"
        config_str = \
            "[PLOTTING]\ncomparison_mode: abs"
        self.shared_valid('comparison_mode', set_option, config_str)

    def test_minimizer_comparison_mode_invalid(self):
        """
        Checks user set comparison_mode is invalid
        """
        config_str = \
            "[PLOTTING]\ncomparison_mode: absolute_values"
        self.shared_invalid('comparison_mode', config_str)

    def test_minimizer_table_type_valid(self):
        """
        Checks user set table_type is valid
        """
        set_option = ['acc', 'compare']
        config_str = \
            "[PLOTTING]\ntable_type: acc\n compare"
        self.shared_valid('table_type', set_option, config_str)

    def test_minimizer_table_type_invalid(self):
        """
        Checks user set table_type is invalid
        """
        config_str = \
            "[PLOTTING]\ntable_type: chi_sq\n "
        self.shared_invalid('table_type', config_str)

    def test_minimizer_results_dir_valid(self):
        """
        Checks user set results_dir is valid
        """
        set_option = os.path.abspath("results_dir")
        config_str = \
            "[PLOTTING]\nresults_dir: results_dir"
        self.shared_valid('results_dir', set_option, config_str)



"""
Tests for the performance profiler file.
"""
from __future__ import absolute_import, division, print_function

import unittest
from collections import OrderedDict

import numpy as np

from fitbenchmarking.core.results_output import preproccess_data
from fitbenchmarking.jacobian.SciPyFD_2point_jacobian import ScipyTwoPoint
from fitbenchmarking.parsing.fitting_problem import FittingProblem
from fitbenchmarking.results_processing import performance_profiler
from fitbenchmarking.utils.fitbm_result import FittingResult
from fitbenchmarking.utils.options import Options


class PerformanceProfillerTests(unittest.TestCase):
    """
    General tests for the performance profiler code.
    """

    def generate_mock_results(self):
        """
        Generates results to test against

        :return: A list of results objects along with expected values for
                 normallised accuracy and runtimes
        :rtype: tuple(list of FittingResults,
                      list of list of float,
                      list of list of float)
        """
        self.num_problems = 4
        self.num_minimizers = 2
        results = []
        options = Options()
        problem = FittingProblem(options)
        problem.starting_values = [{'a': 1, 'b': 2, 'c': 3}]

        acc_in = [[1, 5],
                  [7, 3],
                  [10, 8],
                  [2, 3]]

        runtime_in = [[float('Inf'), 2.2e-3],
                      [3.0e-10, 5.0e-14],
                      [6.9e-7, 4.3e-5],
                      [1.6e-13, 1.8e-13]]

        acc_expected = []
        runtime_expected = []
        for i in range(self.num_problems):
            acc_results = acc_in[i][:]
            acc_expected.append(list(acc_results) / np.min(acc_results))

            runtime_results = runtime_in[i][:]
            runtime_expected.append(
                list(runtime_results) / np.min(runtime_results))
            prob_results = []
            jac = ScipyTwoPoint(problem)
            for j in range(self.num_minimizers):
                minimizer = 'min_{}'.format(j)
                prob_results.append(FittingResult(options=options,
                                                  problem=problem,
                                                  jac=jac,
                                                  initial_params=[1, 2, 3],
                                                  params=[1, 2, 3],
                                                  chi_sq=acc_results[j],
                                                  runtime=runtime_results[j],
                                                  minimizer=minimizer))
            results.append(prob_results)
        return results, acc_expected, runtime_expected

    def setUp(self):
        self.results, self.acc_expected, self.runtime_expected = \
            self.generate_mock_results()
        _ = preproccess_data(self.results)
        self.fig_dir = ''

    def test_correct_prepare_profile_data(self):
        """
        Test that prepare profile data gives the correct result
        """
        acc, runtime = performance_profiler.prepare_profile_data(self.results)
        acc_expected = np.array(self.acc_expected).T
        runtime_expected = np.array(self.runtime_expected).T
        acc_dict = OrderedDict()
        runtime_dict = OrderedDict()
        for j in range(self.num_minimizers):
            acc_dict['min_{}'.format(j)] = acc_expected[j]
            runtime_dict['min_{}'.format(j)] = runtime_expected[j]
        for k, v in acc_dict.items():
            assert np.allclose(v, acc[k])
        for k, v in runtime_dict.items():
            assert np.allclose(v, runtime[k])

    def test_correct_profile(self):
        """
        Test that th performance profiler returns the expected paths
        """
        acc, runtime = performance_profiler.profile(self.results,
                                                    self.fig_dir)

        assert acc == "acc_profile.png"
        assert runtime == "runtime_profile.png"


if __name__ == "__main__":
    unittest.main()



"""
This file holds an exception handler decorator that should wrap all cli
functions to provide cleaner output.
"""

from functools import wraps
import sys

from fitbenchmarking.utils import exceptions
from fitbenchmarking.utils.log import get_logger

LOGGER = get_logger()


def exception_handler(f):
    """
    Decorator to simplify handling exceptions within FitBenchmarking
    This will strip off any 'debug' inputs.

    :param f: The function to wrap
    :type f: python function
    """
    @wraps(f)
    def wrapped(*args, **kwargs):
        debug = kwargs.get('debug', False)

        try:
            return f(*args, **kwargs)
        except exceptions.FitBenchmarkException as e:

            LOGGER.error('Error while running FitBenchmarking. Exiting. '
                         'See below for more information.')
            if debug:
                raise
            else:
                LOGGER.error(str(e))
                sys.exit(1)
        except Exception as e:
            LOGGER.error('Unknown exception. Exiting.')
            if debug:
                raise
            else:
                LOGGER.error(str(e))
                sys.exit(1)

    return wrapped



"""
Runtime table
"""
import numpy as np
import os
from fitbenchmarking.results_processing.base_table import Table


class RuntimeTable(Table):
    """

    The timing results are calculated from an average using the
    `timeit <https://docs.python.org/2/library/timeit.html>`_  module
    in python. The number of runtimes can be set in :ref:`options`.

    """

    def __init__(self, results, best_results, options, group_dir,
                 pp_locations, table_name):
        """
        Initialise the runtime table which shows the average runtime results

        :param results: results nested array of objects
        :type results: list of list of
                       fitbenchmarking.utils.fitbm_result.FittingResult
        :param best_results: best result for each problem
        :type best_results: list of
                        fitbenchmarking.utils.fitbm_result.FittingResult
        :param options: Options used in fitting
        :type options: utils.options.Options
        :param group_dir: path to the directory where group results should be
                          stored
        :type group_dir: str
        :param pp_locations: tuple containing the locations of the
                             performance profiles (acc then runtime)
        :type pp_locations: tuple(str,str)
        :param table_name: Name of the table
        :type table_name: str
        """

        self.name = 'runtime'
        super(RuntimeTable, self).__init__(results, best_results, options,
                                           group_dir, pp_locations, table_name)

        self.has_pp = True
        self.pp_filenames = [os.path.relpath(self.pp_locations[1],group_dir)]

    def get_values(self, results_dict):
        """
        Gets the main values to be reported in the tables

        :param results_dict: dictionary containing results where the keys
                             are the problem sets and the values are lists
                             of results objects
        :type results_dict: dictionary

        :return: two dictionaries containing the absolute runtime and the
                 normalised runtime with respect to the quickest time.
        :rtype: tuple(dict, dict)
        """
        abs_value = {}
        rel_value = {}
        for key, value in results_dict.items():
            abs_value[key] = [v.runtime for v in value]
            min_value = np.min(abs_value[key])
            rel_value[key] = [v.runtime / min_value for v in value]

        return abs_value, rel_value



"""
Functions that create the tables, support pages, figures, and indexes.
"""

from __future__ import (absolute_import, division, print_function)
import inspect
import os
from jinja2 import Environment, FileSystemLoader
from shutil import copy2

import fitbenchmarking
from fitbenchmarking.results_processing import performance_profiler, plots, \
    support_page, tables
from fitbenchmarking.utils import create_dirs
from fitbenchmarking.utils.misc import get_css, get_js


def save_results(options, results, group_name,
                 failed_problems, unselected_minimzers):
    """
    Create all results files and store them.
    Result files are plots, support pages, tables, and index pages.

    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param results: results nested array of objects
    :type results: list of list of
                    fitbenchmarking.utils.fitbm_result.FittingResult
    :param group_name: name of the problem group
    :type group_name: str
    :param failed_problems: list of failed problems to be reported in the
                            html output
    :type failed_problems: list
    :params unselected_minimzers: Dictionary containing unselected minimizers
                                  based on the algorithm_type option
    :type unselected_minimzers: dict

    :return: Path to directory of group results
    :rtype: str
    """
    group_dir, supp_dir, fig_dir, local_css_dir = \
                                        create_directories(options, group_name)

    # copy the template css files into a subfolder of results
    root = os.path.dirname(inspect.getfile(fitbenchmarking))
    template_dir = os.path.join(root, 'templates')
    local_css_dir = os.path.join(options.results_dir, 'css')
    for css_file in ["main_style", "custom_style", "table_style"]:
        copy2(os.path.join(template_dir, css_file + ".css"), local_css_dir)
    best_results = preproccess_data(results)
    pp_locations = performance_profiler.profile(results, fig_dir)

    if options.make_plots:
        create_plots(options, results, best_results, fig_dir)
    support_page.create(options=options,
                        results_per_test=results,
                        group_name=group_name,
                        support_pages_dir=supp_dir)

    table_names, table_descriptions = \
        tables.create_results_tables(options,
                                     results,
                                     best_results,
                                     group_name,
                                     group_dir,
                                     pp_locations,
                                     failed_problems,
                                     unselected_minimzers)

    create_problem_level_index(options,
                               table_names,
                               group_name,
                               group_dir,
                               table_descriptions)

    return group_dir


def create_directories(options, group_name):
    """
    Create the directory structure ready to store the results

    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param group_name: name of the problem group
    :type group_name: str
    :return: paths to the top level results, group results, support pages,
             and figures directories
    :rtype: (str, str, str, str)
    """
    results_dir = create_dirs.results(options.results_dir)
    group_dir = create_dirs.group_results(results_dir, group_name)
    support_dir = create_dirs.support_pages(group_dir)
    figures_dir = create_dirs.figures(support_dir)
    local_css_dir = create_dirs.css(options.results_dir)
    return group_dir, support_dir, figures_dir, local_css_dir


def preproccess_data(results_per_test):
    """
    Preprocess data into the right format for printing and find the best result
    for each problem

    :param results_per_test: results nested array of objects
    :type results_per_test: list of list of
                            fitbenchmarking.utils.fitbm_result.FittingResult

    :return: The best result for each problem
    :rtype: list of fitbenchmarking.utils.fitbm_result.FittingResult
    """
    output = []
    for results in results_per_test:
        best_result = min(results, key=lambda x: x.chi_sq)
        best_result.is_best_fit = True

        min_chi_sq = best_result.chi_sq
        min_runtime = min([r.runtime for r in results])
        for r in results:
            r.min_chi_sq = min_chi_sq
            r.min_runtime = min_runtime
        output.append(best_result)
    return output


def create_plots(options, results, best_results, figures_dir):
    """
    Create a plot for each result and store in the figures directory

    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param results: results nested array of objects
    :type results: list of list of
                   fitbenchmarking.utils.fitbm_result.FittingResult
    :param best_results: best result for each problem
    :type best_results: list of
                        fitbenchmarking.utils.fitbm_result.FittingResult
    :param figures_dir: Path to directory to store the figures in
    :type figures_dir: str
    """
    for best, prob_result in zip(best_results, results):

        plot = plots.Plot(best_result=best,
                          options=options,
                          figures_dir=figures_dir)

        # Create a plot showing the initial guess and get filename
        initial_guess_path = plot.plot_initial_guess()

        # Setup best plot first
        # If none of the fits succeeded, params could be None
        # Otherwise, add the best fit to the plot
        if best.params is not None:
            plot_path = plot.plot_best(best.sanitised_min_name, best.params)
            best.figure_link = plot_path
        else:
            best.figure_error = 'Minimizer failed to produce any parameters'
        best.start_figure_link = initial_guess_path

        # For each result, if it succeeded, create a plot and add plot links to
        # the resuts object
        for result in prob_result:
            # Don't plot best again
            if not result.is_best_fit:
                if result.params is not None:
                    plot_path = plot.plot_fit(result.sanitised_min_name,
                                              result.params)
                    result.figure_link = plot_path
                else:
                    result.figure_error = 'Minimizer failed to produce any ' \
                        'parameters'
                result.start_figure_link = initial_guess_path


def create_problem_level_index(options, table_names, group_name,
                               group_dir, table_descriptions):
    """
    Generates problem level index page.

    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param table_names: list of table names
    :type table_names: list
    :param group_name: name of the problem group
    :type group_name: str
    :param group_dir: Path to the directory where the index should be stored
    :type group_dir: str
    :param table_descriptions: dictionary containing descriptions of the
                               tables and the comparison mode
    :type table_descriptions: dict
    """
    root = os.path.dirname(inspect.getfile(fitbenchmarking))
    template_dir = os.path.join(root, 'templates')
    env = Environment(loader=FileSystemLoader(template_dir))
    css = get_css(options,group_dir)
    js = get_js(options,group_dir)
    template = env.get_template("problem_index_page.html")
    output_file = os.path.join(group_dir, '{}_index.html'.format(group_name))
    links = [v + "html" for v in table_names.values()]
    names = table_names.keys()
    description = [table_descriptions[n] for n in names]
    index = table_descriptions[options.comparison_mode]
    with open(output_file, 'w', encoding="utf-8") as fh:
        fh.write(template.render(
            css_style_sheet=css['main'],
            custom_style=css['custom'],
            mathjax=js['mathjax'],
            group_name=group_name,
            index=index,
            table_type=names,
            links=links,
            description=description,
            zip=zip))



import pytest


def pytest_addoption(parser):
    """
    Command line input function which requires 'all', 'default' or 'external'
    as an input
    """
    parser.addoption(
        "--test-type", action="store", default="all", type=str,
        choices=['all', 'default']
    )


def pytest_configure(config):
    """
    Sets variable `pytest.test_type` to be used within testing
    """
    pytest.test_type = config.getoption('--test-type')



from __future__ import absolute_import, division, print_function

import platform
import sys
import unittest

from fitbenchmarking.utils.options import Options
from fitbenchmarking.utils.output_grabber import OutputGrabber


class OutputGrabberTests(unittest.TestCase):

    def setUp(self):
        self.options = Options()
        self.plt = platform.system()

    def test_correct_stdout(self):
        """
        Test that the OutputGrabber grabs the stdout stream.
        """
        output_string = 'This is the correct output string\nSecond line'
        error_string = 'An error has occurred:\nSome details'

        output = OutputGrabber(self.options)
        with output:
            print(output_string, end='')
            print(error_string, end='', file=sys.stderr)

        # The output grabber is not enabled for windows
        if self.plt != "Windows":
            assert output.stdout_grabber.capturedtext == output_string

    def test_correct_stderr(self):
        """
        Test that the OutputGrabber grabs the stderr stream.
        """
        output_string = 'This is the correct output string\nSecond line'
        error_string = 'An error has occurred:\nSome details'

        output = OutputGrabber(self.options)
        with output:
            print(output_string, end='')
            print(error_string, end='', file=sys.stderr)

        # The output grabber is not enabled for windows
        if self.plt != "Windows":
            assert output.stderr_grabber.capturedtext == error_string


if __name__ == "__main__":
    unittest.main()



"""
Set up and build the support pages for various types of problems.
"""

from __future__ import (absolute_import, division, print_function)

import inspect
import os

from jinja2 import Environment, FileSystemLoader

import fitbenchmarking
from fitbenchmarking.utils.misc import get_css


def create(results_per_test, group_name, support_pages_dir,
           options):
    """
    Iterate through problem results and create a support html page for
    each.

    :param results_per_test: results object
    :type results_per_test: list[list[list]]
    :param group_name: name of the problem group
    :type group_name: str
    :param support_pages_dir: directory in which the results are saved
    :type support_pages_dir: str
    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    """

    for prob_result in results_per_test:

        create_prob_group(prob_result,
                          group_name,
                          support_pages_dir,
                          options)


def create_prob_group(prob_results, group_name, support_pages_dir,
                      options):
    """
    Creates a support page containing figures and other
    details about the fit for a problem.
    A link to the support page is stored in the results object.

    :param prob_results: problem results objects containing results for
                         each minimizer and a certain fitting function
    :type prob_results: list[fitbenchmarking.utils.fitbm_result.FittingResult]
    :param group_name: name of the problem group
    :type group_name: str
    :param support_pages_dir: directory to store the support pages in
    :type support_pages_dir: str
    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    """

    for result in prob_results:
        prob_name = result.sanitised_name

        file_name = '{}_{}_{}.html'.format(
            group_name, prob_name, result.sanitised_min_name)
        file_name = file_name.lower()
        file_path = os.path.join(support_pages_dir, file_name)

        # Bool for print message/insert image
        fit_success = init_success = options.make_plots

        if options.make_plots:
            fig_fit, fig_start = get_figure_paths(result)
            if fig_fit == '':
                fig_fit = result.figure_error
                fit_success = False
            if fig_start == '':
                fig_start = result.figure_error
                init_success = False
        else:
            fig_fit = fig_start = 'Re-run with make_plots set to yes in the ' \
                                  'ini file to generate plots.'

        root = os.path.dirname(inspect.getfile(fitbenchmarking))
        template_dir = os.path.join(root, "templates")
        env = Environment(loader=FileSystemLoader(template_dir))
        css = get_css(options, support_pages_dir)
        template = env.get_template("support_page_template.html")

        with open(file_path, 'w') as fh:
            fh.write(template.render(
                css_style_sheet=css['main'],
                table_style=css['table'],
                custom_style=css['custom'],
                title=result.name,
                equation=result.problem.equation,
                initial_guess=result.ini_function_params,
                minimizer=result.minimizer,
                is_best_fit=result.is_best_fit,
                initial_plot_available=init_success,
                initial_plot=fig_start,
                min_params=result.fin_function_params,
                fitted_plot_available=fit_success,
                fitted_plot=fig_fit))

        result.support_page_link = os.path.relpath(file_path)


def get_figure_paths(result):
    """
    Get the paths to the figures used in the support page.

    :param result: The result to get the figures for
    :type result: fitbenchmarking.utils.fitbm_result.FittingProblem

    :return: the paths to the required figures
    :rtype: tuple(str, str)
    """

    figures_dir = "figures"

    output = []
    for link in [result.figure_link, result.start_figure_link]:
        if link == '':
            output.append('')
        else:
            path = os.path.join(figures_dir, link)
            output.append(path)

    return output[0], output[1]



"""
This is the main entry point into the FitBenchmarking software package.
For more information on usage type fitbenchmarking --help
or for more general information, see the online docs at
docs.fitbenchmarking.com.
"""

from __future__ import absolute_import, division, print_function

import argparse
from distutils.dir_util import copy_tree
import glob
import inspect
import os
import tempfile
import sys
import webbrowser

from jinja2 import Environment, FileSystemLoader

import fitbenchmarking
from fitbenchmarking.cli.exception_handler import exception_handler
from fitbenchmarking.core.fitting_benchmarking import benchmark
from fitbenchmarking.core.results_output import save_results
from fitbenchmarking.utils.exceptions import OptionsError
from fitbenchmarking.utils.log import get_logger, setup_logger
from fitbenchmarking.utils.misc import get_css
from fitbenchmarking.utils.options import Options

LOGGER = get_logger()


def get_parser():
    """
    Creates and returns a parser for the args.

    :return: configured argument parser
    :rtype: argparse.ArgParser
    """

    epilog = '''Usage Examples:

    $ fitbenchmarking
    $ fitbenchmarking -p examples/benchmark_problems/NIST/*
    $ fitbenchmarking -o examples/options_template.ini \
-p examples/benchmark_problems/simple_tests examples/benchmark_problems/Muon '''

    parser = argparse.ArgumentParser(
        prog='FitBenchmarking', add_help=True, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    root = os.path.dirname(inspect.getfile(fitbenchmarking))

    parser.add_argument('-o', '--options-file',
                        metavar='OPTIONS_FILE',
                        default='',
                        help='The path to a %(prog)s options file')
    parser.add_argument('-p', '--problem_sets',
                        nargs='+',
                        default=glob.glob(os.path.join(root,
                                                       'benchmark_problems',
                                                       'NIST',
                                                       '*')),
                        help='Paths to directories containing problem sets.')
    parser.add_argument('-d', '--debug-mode',
                        default=False,
                        action='store_true',
                        help='Enable debug mode (prints traceback)',)

    return parser


@exception_handler
def run(problem_sets, options_file='', debug=False):
    """
    Run benchmarking for the problems sets and options file given.
    Opens a webbrowser to the results_index after fitting.

    :param problem_sets: The paths to directories containing problem_sets
    :type problem_sets: list of str
    :param options_file: The path to an options file, defaults to ''
    :type options_file: str, optional
    :param debug: Enable debugging output
    :type debug: bool
    """
    # Find the options file
    current_path = os.path.abspath(os.path.curdir)
    if options_file != '':
        # Read custom minimizer options from file
        glob_options_file = glob.glob(options_file)
        if glob_options_file == []:
            raise OptionsError('Could not find file {}'.format(options_file))
        if not options_file.endswith(".ini"):
            raise OptionsError('Options file must be a ".ini" file')
        else:
            options = Options(glob_options_file)
    else:
        options = Options()

    setup_logger(log_file=options.log_file,
                 append=options.log_append,
                 level=options.log_level)

    opt_file = tempfile.NamedTemporaryFile(suffix='.ini',
                                           mode='w',
                                           delete=False)
    options.write_to_stream(opt_file)
    opt_file.close()
    LOGGER.debug("The options file used is as follows:")
    with open(opt_file.name) as f:
        for line in f:
            LOGGER.debug(line.replace("\n", ""))
    os.remove(opt_file.name)

    groups = []
    result_dir = []
    for sub_dir in problem_sets:

        # Create full path for the directory that holds a group of
        # problem definition files
        data_dir = os.path.join(current_path, sub_dir)

        test_data = glob.glob(data_dir + '/*.*')

        if test_data == []:
            LOGGER.warning('Problem set %s not found', data_dir)
            continue

        # generate group label/name used for problem set
        try:
            with open(os.path.join(data_dir, 'META.txt'), 'r') as f:
                label = f.readline().strip('\n')
        except IOError:
            label = sub_dir.replace('/', '_')

        LOGGER.info('Running the benchmarking on the %s problem set',
                    label)
        results, failed_problems, unselected_minimzers = \
            benchmark(options=options,
                      data_dir=data_dir)
        LOGGER.info('Producing output for the %s problem set', label)
        # Display the runtime and accuracy results in a table
        group_results_dir = \
            save_results(group_name=label,
                         results=results,
                         options=options,
                         failed_problems=failed_problems,
                         unselected_minimzers=unselected_minimzers)

        LOGGER.info('Completed benchmarking for %s problem set', sub_dir)
        group_results_dir = os.path.relpath(path=group_results_dir,
                                            start=options.results_dir)
        result_dir.append(group_results_dir)
        groups.append(label)

        # resets options to original values
        options.reset()
    if os.path.basename(options.results_dir) == \
            options.DEFAULT_PLOTTING['results_dir']:
        LOGGER.info("\nWARNING: \nThe FitBenchmarking results will be "
                    "placed into the folder: \n\n   {}\n\nTo change this "
                    "alter the input options "
                    "file.\n".format(options.results_dir))

    root = os.path.dirname(inspect.getfile(fitbenchmarking))
    template_dir = os.path.join(root, 'templates')
    env = Environment(loader=FileSystemLoader(template_dir))
    css = get_css(options, options.results_dir)
    template = env.get_template("index_page.html")
    group_links = [os.path.join(d, "{}_index.html".format(g))
                   for g, d in zip(groups, result_dir)]
    output_file = os.path.join(options.results_dir, 'results_index.html')

    # Copying fonts directory into results directory
    copy_tree(os.path.join(root, 'fonts'),
              os.path.join(options.results_dir, "fonts"))
    # Copying js directory into results directory
    copy_tree(os.path.join(template_dir,'js'),
              os.path.join(options.results_dir, "js"))
    
    with open(output_file, 'w') as fh:
        fh.write(template.render(
            css_style_sheet=css['main'],
            custom_style=css['custom'],
            groups=groups,
            group_link=group_links,
            zip=zip))
    webbrowser.open_new(output_file)


def main():
    """
    Entry point to be exposed as the `fitbenchmarking` command.
    """
    parser = get_parser()

    if len(sys.argv) == 1:
        print("Running with NIST problem set")

    args = parser.parse_args(sys.argv[1:])

    run(problem_sets=args.problem_sets,
        options_file=args.options_file, debug=args.debug_mode)


if __name__ == '__main__':
    main()



"""
Functions to update style sheet from fitbenchmarking.com
"""
try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest
import inspect
import os
import wget

import fitbenchmarking

root = os.path.dirname(inspect.getfile(fitbenchmarking))
template_dir = os.path.join(root, 'templates')
filename = wget.download(
    "https://fitbenchmarking.com/assets/css/main.css", template_dir)

with open(filename, 'r') as f:
    expected = f.readlines()

style_sheet = os.path.join(template_dir, "main_style.css")
with open(style_sheet, 'r') as f:
    actual = f.readlines()

diff = []
for exp_line, act_line in zip_longest(expected, actual):
    if exp_line != act_line:
        diff.append([exp_line, act_line])

if diff == []:
    print("\nCurrent style sheet is up to date")
    os.remove(filename)
else:
    print("\nCurrent style sheet is out of date so downloading from "
          "fitbenchmarking.com")
    os.remove(style_sheet)
    os.rename(filename, style_sheet)



'''
Test the LOGGING section for the options file
'''
import inspect
import shutil
import os
import unittest

from fitbenchmarking.utils import exceptions
from fitbenchmarking.utils.options import Options


class LoggingOptionTests(unittest.TestCase):
    """
    Checks the logging defaults are set correctly
    """

    def setUp(self):
        """
        Initializes options class with defaults
        """
        self.options = Options()

    def test_file_name_default(self):
        """
        Checks file_name default
        """
        expected = 'fitbenchmarking.log'
        actual = self.options.log_file
        self.assertEqual(expected, actual)

    def test_append_default(self):
        """
        Checks append default
        """
        expected = False
        actual = self.options.log_append
        self.assertEqual(expected, actual)

    def test_level_default(self):
        """
        Checks level default
        """
        expected = 'INFO'
        actual = self.options.log_level
        self.assertEqual(expected, actual)

    def test_external_output_default(self):
        """
        Checks external_output default
        """
        expected = 'log_only'
        actual = self.options.external_output
        self.assertEqual(expected, actual)


class UserLoggingOptionTests(unittest.TestCase):
    """
    Checks the logging in the options file are set correctly or raise errors
    """

    def setUp(self):
        """
        Sets the directory to save the temporary ini files in
        """
        options_dir = os.path.dirname(inspect.getfile(Options))
        self.test_files_dir = os.path.join(options_dir, 'tests', 'files')
        os.mkdir(self.test_files_dir)

    def tearDown(self):
        """
        Deletes temporary folder and results produced
        """
        if os.path.exists(self.test_files_dir):
            shutil.rmtree(self.test_files_dir)

    def generate_user_ini_file(self, opt_name, config_str):
        """
        Generates user defined ini file

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: section of an ini file which sets the option
        :type config_str: str

        :return: location of temporary ini file
        :rtype: str
        """
        opts_file = os.path.join(self.test_files_dir,
                                 'test_{}_valid.ini'.format(opt_name))
        with open(opts_file, 'w') as f:
            f.write(config_str)
        return opts_file

    def shared_valid(self, opt_name, options_set, config_str):
        """
        Shared test to check that the logging option set is valid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param options_set: option set to be tested
        :type options_set: list
        :param config_str: section of an ini file which sets the option
        :type config_str: str
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        options = Options(opts_file)
        actual = getattr(options, opt_name)
        self.assertEqual(options_set, actual)

    def shared_invalid(self, opt_name, config_str):
        """
        Shared test to check that the logging option set is invalid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: option set to be tested
        :type config_str: list
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        with self.assertRaises(exceptions.OptionsError):
            Options(opts_file)

    def test_invalid_option_key(self):
        """
        Tests that the user defined option key is invalid.
        """
        config_str = \
            "[LOGGING]\nlog_level: DEBUG"
        self.shared_invalid('log_level', config_str)

    def test_minimizer_file_name_valid(self):
        """
        Checks user set file_name is valid
        """
        set_option = "fitb.log"
        config_str = \
            "[LOGGING]\nfile_name: fitb.log"
        self.shared_valid('log_file', set_option, config_str)

    def test_minimizer_log_append_valid(self):
        """
        Checks user set log_append is valid
        """
        set_option = True
        config_str = \
            "[LOGGING]\nappend: yes"
        self.shared_valid('log_append', set_option, config_str)

    def test_minimizer_log_append_invalid(self):
        """
        Checks user set log_append are invalid
        """
        config_str = \
            "[LOGGING]\nappend: append_to_bottom"
        self.shared_invalid('log_append', config_str)

    def test_minimizer_log_level_valid(self):
        """
        Checks user set log_level is valid
        """
        set_option = "DEBUG"
        config_str = \
            "[LOGGING]\nlevel: DEBUG"
        self.shared_valid('log_level', set_option, config_str)

    def test_minimizer_log_level_invalid(self):
        """
        Checks user set log_level is invalid
        """
        config_str = \
            "[LOGGING]\nlevel: debug"
        self.shared_invalid('log_level', config_str)

    def test_minimizer_external_output_valid(self):
        """
        Checks user set external_output is valid
        """
        set_option = 'debug'
        config_str = \
            "[LOGGING]\nexternal_output: debug"
        self.shared_valid('external_output', set_option, config_str)

    def test_minimizer_external_output_invalid(self):
        """
        Checks user set external_output is invalid
        """
        config_str = \
            "[LOGGING]\nexternal_output: remove_third_party_output"
        self.shared_invalid('external_output', config_str)



"""
Set up and build the results tables.
"""

from __future__ import (absolute_import, division, print_function)
import copy
from importlib import import_module
from inspect import getfile, getmembers, isabstract, isclass
import os
from jinja2 import Environment, FileSystemLoader

import fitbenchmarking
from fitbenchmarking.results_processing.base_table import Table
from fitbenchmarking.utils.exceptions import UnknownTableError
from fitbenchmarking.utils.misc import get_css, get_js

ERROR_OPTIONS = {0: "Successfully converged",
                 1: "Software reported maximum number of iterations exceeded",
                 2: "Software run but didn't converge to solution",
                 3: "Software raised an exception"}

SORTED_TABLE_NAMES = ["compare", "acc", "runtime", "local_min"]


def create_results_tables(options, results, best_results, group_name,
                          group_dir, pp_locations, failed_problems,
                          unselected_minimzers):
    """
    Saves the results of the fitting to html/txt tables.

    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param results: results nested array of objects
    :type results: list of list of
                   fitbenchmarking.utils.fitbm_result.FittingResult
    :param best_results: best result for each problem
    :type best_results: list of
                        fitbenchmarking.utils.fitbm_result.FittingResult
    :param group_name: name of the problem group
    :type group_name: str
    :param group_dir: path to the directory where group results should be
                      stored
    :type group_dir: str
    :param table_descriptions: dictionary containing descriptions of the
                               tables and the comparison mode
    :type table_descriptions: dict
    :param pp_locations: tuple containing the locations of the
                         performance profiles (acc then runtime)
    :type pp_locations: tuple(str,str)
    :param failed_problems: list of failed problems to be reported in the
                            html output
    :type failed_problems: list
    :params unselected_minimzers: Dictionary containing unselected minimizers
                                  based on the algorithm_type option
    :type unselected_minimzers: dict

    :return: filepaths to each table
             e.g {'acc': <acc-table-filename>, 'runtime': ...}
             and dictionary of table descriptions
    :rtype: tuple(dict, dict)
    """

    weighted_str = 'weighted' if options.use_errors else 'unweighted'

    table_names = {}
    description = {}
    for suffix in SORTED_TABLE_NAMES:
        if suffix in options.table_type:
            table_names[suffix] = '{0}_{1}_{2}_table.'.format(group_name,
                                                              suffix,
                                                              weighted_str)

            table, html_table, txt_table = generate_table(results,
                                                          best_results,
                                                          options,
                                                          group_dir,
                                                          pp_locations,
                                                          table_names[suffix],
                                                          suffix)

            table_title = table.table_title
            file_path = table.file_path

            description = table.get_description(description)

            table_format = None if suffix == 'local_min' \
                else description[options.comparison_mode]

            has_pp = table.has_pp

            pp_filenames = table.pp_filenames

            root = os.path.dirname(getfile(fitbenchmarking))
            template_dir = os.path.join(root, 'templates')
            css = get_css(options, group_dir)
            js = get_js(options, group_dir)
            env = Environment(loader=FileSystemLoader(template_dir))
            template = env.get_template("table_template.html")
            html_output_file = file_path + 'html'
            txt_output_file = file_path + 'txt'

            with open(txt_output_file, "w") as f:
                f.write(txt_table)
            failed_minimzers = sum(list(unselected_minimzers.values()), [])
            report_failed_min = failed_minimzers != []

            with open(html_output_file, "w", encoding="utf-8") as f:
                f.write(
                    template.render(css_style_sheet=css['main'],
                                    custom_style=css['custom'],
                                    table_style=css['table'],
                                    mathjax=js['mathjax'],
                                    table_description=description[suffix],
                                    table_format=table_format,
                                    result_name=table_title,
                                    has_pp=has_pp,
                                    pp_filenames=pp_filenames,
                                    table=html_table,
                                    error_message=ERROR_OPTIONS,
                                    failed_problems=failed_problems,
                                    unselected_minimzers=unselected_minimzers,
                                    algorithm_type=options.algorithm_type,
                                    report_failed_min=report_failed_min))

    return table_names, description


def load_table(table):
    """
    Create and return table object.

    :param table: The name of the table to create a table for
    :type table: string

    :return: Table class for the problem
    :rtype: fitbenchmarking/results_processing/tables.Table subclass
    """

    module_name = '{}_table'.format(table.lower())

    try:
        module = import_module('.' + module_name, __package__)
    except ImportError as e:
        raise UnknownTableError('Given table option {} '
                                'was not found: {}'.format(table, e))

    classes = getmembers(module, lambda m: (isclass(m)
                                            and not isabstract(m)
                                            and issubclass(m, Table)
                                            and m is not Table))
    return classes[0][1]


def generate_table(results, best_results, options, group_dir,
                   pp_locations, table_name, suffix):
    """
    Generate html/txt tables.

    :param results: results nested array of objects
    :type results: list of list of
                   fitbenchmarking.utils.fitbm_result.FittingResult
    :param best_results: best result for each problem
    :type best_results: list of
                        fitbenchmarking.utils.fitbm_result.FittingResult
    :param options: The options used in the fitting problem and plotting
    :type options: fitbenchmarking.utils.options.Options
    :param group_dir: path to the directory where group results should be
                      stored
    :type group_dir: str
    :param pp_locations: tuple containing the locations of the
                         performance profiles (acc then runtime)
    :type pp_locations: tuple(str,str)
    :param table_name: name of the table
    :type table_name: str
    :param suffix: table suffix
    :type suffix: str


    :return: Table object, HTML string of table and text string of table.
    :rtype: tuple(Table object, str, str)
    """
    table_module = load_table(suffix)
    table = table_module(results, best_results,
                         options, group_dir,
                         pp_locations, table_name)

    results_dict = table.create_results_dict()

    disp_results = table.get_values(results_dict)
    error = table.get_error(results_dict)
    links = table.get_links(results_dict)
    colour = table.get_colour(disp_results)
    str_results = table.display_str(disp_results)

    pandas_html = table.create_pandas_data_frame(str_results)
    pandas_txt = copy.copy(pandas_html)

    html_table = table.to_html(pandas_html, colour, links, error)
    txt_table = table.to_txt(pandas_txt, error)
    return table, html_table, txt_table



"""
Module which allows access to variables in the code
"""

from pprint import pformat
from importlib import import_module
from docutils import nodes
from docutils.parsers.rst import Directive, directives


class PrettyPrintModuleValue(Directive):
    """
    Class which enables documentation to access variables from code base
    """

    has_content = False
    optional_arguments = 2
    option_spec = {
        'module': directives.unchanged_required,
        'var': directives.unchanged_required,
    }

    def run(self):
        """
        Accesses variable
        """
        inp_module = self.options['module']
        inp_var = self.options['var']

        # Get variable value
        module = import_module(inp_module)
        value = getattr(module, inp_var)
        value = pformat(value, 2, width=68)

        paragraph_node = nodes.literal_block(text=value)
        return [paragraph_node]


def setup(app):
    """
    Setup PrettyPrintModuleValue for use in docs
    """
    app.add_directive("prettyprintmodulevalue", PrettyPrintModuleValue)

    return {
        'version': '0.1',
        'parallel_read_safe': True,
        'parallel_write_safe': True,
    }



"""
This file contains tests for the parsers.
"""

from importlib import import_module
from inspect import isclass, isabstract, getmembers
from json import load
import os
from unittest import TestCase
from pytest import test_type as TEST_TYPE

import numpy as np

from fitbenchmarking.parsing.base_parser import Parser
from fitbenchmarking.parsing.fitting_problem import FittingProblem
from fitbenchmarking.parsing.parser_factory import \
    ParserFactory, parse_problem_file
from fitbenchmarking.utils import exceptions
from fitbenchmarking.utils.options import Options

OPTIONS = Options()


# pylint: disable=no-self-use
def pytest_generate_tests(metafunc):
    """
    Function used by pytest to parametrize tests.
    This will create a set of tests for each function in the class where
    the parameters are given in a 'params' dict in the class.
    """
    # called once per each test function
    funcarglist = metafunc.cls.params[metafunc.function.__name__]
    argnames = sorted(funcarglist[0])
    argvals = [[funcargs[name]
                for name in argnames]
               for funcargs in funcarglist]
    metafunc.parametrize(argnames,
                         argvals)


def generate_test_cases():
    """
    Utility function to create the params dict for parametrising the tests.

    :return: params dictionary with a function name as a key, and a list of
             parameter dictionaries for the value
    :rtype: dict
    """
    params = {'test_parsers': [],
              'test_factory': [],
              'test_function_evaluation': []}

    # get all parsers
    test_dir = os.path.dirname(__file__)
    if TEST_TYPE == "all":
        formats = ['cutest', 'nist', 'fitbenchmark']
    elif TEST_TYPE == "default":
        formats = ['nist']

    # create list of test_cases
    expected_dir = os.listdir(os.path.join(test_dir, 'expected'))
    for file_format in formats:
        format_dir = os.listdir(os.path.join(test_dir, file_format))
        for expected_file in expected_dir:
            expected_path = os.path.join(test_dir, 'expected', expected_file)
            test_name = os.path.splitext(expected_file)[0]
            test_name_with_ext = [f for f in format_dir
                                  if f.startswith(test_name)]
            if not test_name_with_ext:
                test_file = None
            elif len(test_name_with_ext) == 1:
                test_file = os.path.join(test_dir,
                                         file_format,
                                         test_name_with_ext[0])
            else:
                raise RuntimeError(
                    'Too many "{}" files found for "{}" test'.format(
                        file_format, test_name))

            test_parsers = {}
            test_parsers['file_format'] = file_format
            test_parsers['test_file'] = test_file
            test_parsers['expected'] = load_expectation(expected_path)
            params['test_parsers'].append(test_parsers)

            test_factory = {}
            test_factory['file_format'] = file_format
            test_factory['test_file'] = test_file
            params['test_factory'].append(test_factory)

        func_eval = os.path.join(test_dir,
                                 file_format,
                                 'function_evaluations.json')
        test_func_eval = {}
        test_func_eval['file_format'] = file_format
        test_func_eval['evaluations_file'] = func_eval
        params['test_function_evaluation'].append(test_func_eval)

    return params


def load_expectation(filename):
    """
    Load an expected fitting problem from a json file.

    :param filename: The path to the expectation file
    :type filename: string
    :return: A fitting problem to test against
    :rtype: fitbenchmarking.parsing.FittingProblem
    """
    with open(filename, 'r') as f:
        expectation_dict = load(f)

    expectation = FittingProblem(OPTIONS)
    expectation.name = expectation_dict['name']
    expectation.start_x = expectation_dict['start_x']
    expectation.end_x = expectation_dict['end_x']
    expectation.data_x = np.array(expectation_dict['data_x'])
    expectation.data_y = np.array(expectation_dict['data_y'])
    expectation.data_e = expectation_dict['data_e']
    if expectation.data_e is not None:
        expectation.data_e = np.array(expectation.data_e)
    expectation.starting_values = expectation_dict['starting_values']
    expectation.value_ranges = expectation_dict['value_ranges']

    return expectation


class TestParsers:
    """
    A class to hold the tests for parametrized testing of parsers.
    """

    params = generate_test_cases()

    def test_parsers(self, file_format, test_file, expected):
        """
        Test that the parser correctly parses the specified input file.

        :param file_format: The name of the file format
        :type file_format: string
        :param test_file: The path to the test file
        :type test_file: string
        """
        assert (test_file is not None), \
            'No test file for {}'.format(file_format)

        with open(test_file) as f:
            if f.readline() == 'NA':
                # Test File cannot be written
                return

        # Test import
        module = import_module(name='.{}_parser'.format(file_format),
                               package='fitbenchmarking.parsing')

        parser = getmembers(module, lambda m: (isclass(m)
                                               and not isabstract(m)
                                               and issubclass(m, Parser)
                                               and m is not Parser))[0][1]

        # Test parse
        with parser(test_file, OPTIONS) as p:
            fitting_problem = p.parse()

        # Allow for problems not supporting certain test cases
        # (e.g. value_ranges)
        if fitting_problem.name == 'NA':
            return

        # Check against expected
        fitting_problem.verify()

        # functions and equation need to be done seperately as they can't be
        # generic across problem types.
        # similarly starting_values uses the param name so must be checked
        # separately
        for attr in ['name', 'data_x', 'data_y', 'data_e', 'start_x', 'end_x',
                     'value_ranges']:
            parsed_attr = getattr(fitting_problem, attr)
            expected_attr = getattr(expected, attr)
            equal = (parsed_attr == expected_attr)
            if isinstance(equal, np.ndarray):
                equal = equal.all()
            assert (equal), '{} was parsed incorrectly.'.format(attr) \
                + '{} != {}'.format(parsed_attr, expected_attr)

        # Check starting_values
        for a, e in zip(fitting_problem.starting_values,
                        expected.starting_values):
            loaded_as_set = set(a.values())
            expected_as_set = set(e.values())
            assert (loaded_as_set == expected_as_set), \
                'starting_values were parsed incorrectly.'

        # Check that the function is callable
        assert callable(fitting_problem.function)

    def test_function_evaluation(self, file_format, evaluations_file):
        """
        Tests that the function evaluation is consistent with what would be
        expected by comparing to some precomputed values with fixed params and
        x values.

        :param file_format: The name of the file format
        :type file_format: string
        :param evaluations_file: Path to a json file containing tests and
                                 results
                                 in the following format:
                                 {"test_file1": [[x1, params1, results1],
                                                 [x2, params2, results2],
                                                  ...],
                                  "test_file2": ...}
        :type evaluations_file: string
        """

        assert (evaluations_file is not None), \
            'No function evaluations provided to test against for {}'.format(
                file_format)

        with open(evaluations_file, 'r') as ef:
            results = load(ef)

        format_dir = os.path.dirname(evaluations_file)

        for f, tests in results.items():
            f = os.path.join(format_dir, f)

            parser = ParserFactory.create_parser(f)
            with parser(f, OPTIONS) as p:
                fitting_problem = p.parse()

            for r in tests:
                x = np.array(r[0])
                actual = fitting_problem.eval_f(x=x,
                                                params=r[1])
                print(r, actual)
                assert np.isclose(actual, r[2]).all()

    def test_factory(self, file_format, test_file):
        """
        Tests that the factory selects the correct parser

        :param file_format: The name of the file format
        :type file_format: string
        :param test_file: The path to the test file
        :type test_file: string
        """

        parser = ParserFactory.create_parser(test_file)
        assert (parser.__name__.lower().startswith(file_format.lower())), \
            'Factory failed to get associated parser for {0}: got {1},' \
            'required starting with {1}'.format(test_file,
                                                parser.__name__.lower())


class TestParserFactory(TestCase):
    """
    A class to hold the tests for the parser factory.
    Note: testing that the parser factory works for all new parsers is left to
          the factory tests in TestParsers
    """

    def test_unknown_parser(self):
        """
        Tests that the parser factory raises a NoParserError when an erroneous
        parser is requested.
        """
        filename = os.path.join(os.getcwd(), 'this_is_a_fake_parser.txt')
        with open(filename, 'w') as f:
            f.write('this_is_a_fake_parser')

        factory = ParserFactory()
        with self.assertRaises(exceptions.NoParserError):
            _ = factory.create_parser(filename)

        os.remove(filename)

    def test_parse_problem_file(self):
        """
        Tests the parse_problem_file method
        """
        filename = os.path.join(os.path.dirname(__file__),
                                'nist',
                                'basic.dat')
        fitting_problem = parse_problem_file(filename, OPTIONS)
        self.assertEqual(fitting_problem.name, 'basic')



"""
Implements a controller for the scipy ls fitting software.
In particular, for the scipy least_squares solver.
"""

from scipy.optimize import least_squares

from fitbenchmarking.controllers.base_controller import Controller


class ScipyLSController(Controller):
    """
    Controller for the Scipy Least-Squares fitting software.
    """

    def __init__(self, problem):
        """
        Initialises variable used for temporary storage.
        """
        super(ScipyLSController, self).__init__(problem)

        self._popt = None
        self.algorithm_check = {
            'all': ['lm-scipy-no-jac', 'lm-scipy', 'trf', 'dogbox'],
            'ls': ['lm-scipy-no-jac', 'lm-scipy', 'trf', 'dogbox'],
            'deriv_free': [None],
            'general': [None]}

    def setup(self):
        """
        Setup problem ready to be run with SciPy LS
        """
        if self.minimizer == "lm-scipy":
            self.minimizer = "lm"

    def fit(self):
        """
        Run problem with Scipy LS.
        """
        # The minimizer "lm-scipy-no-jac" uses MINPACK's Jacobian evaluation
        # which are significantly faster and gives different results than
        # using the minimizer "lm-scipy" which uses problem.eval_j for the
        # Jacobian evaluation. We do not see significant speed changes or
        # difference in the accuracy results when running trf or dogbox with
        # or without problem.jac.eval for the Jacobian evaluation
        if self.minimizer == "lm-scipy-no-jac":
            self.result = least_squares(fun=self.problem.eval_r,
                                        x0=self.initial_params,
                                        method="lm",
                                        max_nfev=500)
        else:
            self.result = least_squares(fun=self.problem.eval_r,
                                        x0=self.initial_params,
                                        method=self.minimizer,
                                        jac=self.problem.jac.eval,
                                        max_nfev=500)

        self._popt = self.result.x
        self._status = self.result.status

    def cleanup(self):
        """
        Convert the result to a numpy array and populate the variables results
        will be read from.
        """
        if self._status > 0:
            self.flag = 0
        elif self._status == 0:
            self.flag = 1
        else:
            self.flag = 2

        self.final_params = self._popt



"""
This file contains a factory implementation for the Jacobians.
This is used to manage the imports and reduce effort in adding new Jacobian
methods.
"""
from importlib import import_module
from inspect import getmembers, isabstract, isclass

from fitbenchmarking.jacobian.base_jacobian import Jacobian
from fitbenchmarking.utils.exceptions import NoJacobianError


def create_jacobian(options):
    """
    Create a Jacobian class.

    :param options: FitBenchmarking options object
    :type options: fitbenchmarking.utils.Options

    :return: Controller class for the problem
    :rtype: fitbenchmarking.jacobian.base_controller.Jacobian subclass
    """
    jac_method = options.jac_method
    num_method = options.num_method

    module_name = f'{jac_method}_jacobian' if jac_method == 'analytic' \
        else f'{jac_method}_{num_method}_jacobian'

    try:
        module = import_module('.' + module_name, __package__)
    except ImportError:
        raise NoJacobianError(f'Could not find Jacobian class with type as '
                              f'{jac_method} and numerical method as '
                              f'{num_method}.')

    classes = getmembers(module, lambda m: (isclass(m)
                                            and not isabstract(m)
                                            and issubclass(m, Jacobian)
                                            and m is not Jacobian))

    return classes[0][1]



"""
Main module of the tool, this holds the master function that calls
lower level functions to fit and benchmark a set of problems
for a certain fitting software.
"""

from __future__ import absolute_import, division, print_function

from collections import defaultdict
import copy
import timeit
import warnings
import os

import numpy as np

from fitbenchmarking.controllers.controller_factory import ControllerFactory
from fitbenchmarking.utils.exceptions import NoResultsError, \
    UnknownMinimizerError, UnsupportedMinimizerError, MissingSoftwareError, \
    ControllerAttributeError
from fitbenchmarking.parsing.parser_factory import parse_problem_file
from fitbenchmarking.jacobian.jacobian_factory import create_jacobian
from fitbenchmarking.utils import fitbm_result, misc, output_grabber
from fitbenchmarking.utils.log import get_logger

LOGGER = get_logger()


def benchmark(options, data_dir):
    """
    Gather the user input and list of paths. Call benchmarking on these.
    The benchmarking structure is:

    .. code-block:: python

        loop_over_benchmark_problems()
            loop_over_starting_values()
                loop_over_software()
                    loop_over_minimizers()

    :param options: dictionary containing software used in fitting
                    the problem, list of minimizers and location of
                    json file contain minimizers
    :type options: fitbenchmarking.utils.options.Options
    :param data_dir: full path of a directory that holds a group of problem
                     definition files
    :type date_dir: str

    :return: prob_results array of fitting results for the problem group,
             list of failed problems and dictionary of unselected minimizers
    :rtype: tuple(list, list, dict)
    """

    # Extract problem definitions
    problem_group = misc.get_problem_files(data_dir)

    #################################
    # Loops over benchmark problems #
    #################################
    results, failed_problems, unselected_minimizers = \
        loop_over_benchmark_problems(problem_group, options)

    new_minimizer = {}
    minimizer_copy = copy.copy(options.minimizers)
    for keys, minimizers in unselected_minimizers.items():
        new_minimizer[keys] = [x for x in minimizer_copy[keys]
                               if x not in minimizers]
    options.minimizers = new_minimizer

    # Used to group elements in list by name
    results_dict = defaultdict(list)
    for problem_result in results:
        results_dict[problem_result.name].append(problem_result)

    results = [results_dict[r] for r in
               sorted(results_dict.keys(), key=lambda x: x.lower())]

    return results, failed_problems, unselected_minimizers


def loop_over_benchmark_problems(problem_group, options):
    """
    Loops over benchmark problems

    :param problem_group: locations of the benchmark problem files
    :type problem_group: list
    :param options: FitBenchmarking options for current run
    :type options: fitbenchmarking.utils.options.Options

    :return: prob_results array of fitting results for the problem group,
             list of failed problems and dictionary of unselected minimizers
    :rtype: tuple(list, list, dict)
    """
    grabbed_output = output_grabber.OutputGrabber(options)
    results = []
    failed_problems = []
    for i, p in enumerate(problem_group):
        problem_passed = True
        info_str = " Running data from: {} {}/{}".format(
            os.path.basename(p), i + 1, len(problem_group))

        LOGGER.info('\n' + '#' * (len(info_str) + 1))
        LOGGER.info(info_str)
        LOGGER.info('#' * (len(info_str) + 1))

        try:
            with grabbed_output:
                parsed_problem = parse_problem_file(p, options)
                parsed_problem.correct_data()
        except Exception as e:
            info_str = " Could not run data from: {} {}/{}".format(
                p, i + 1, len(problem_group))
            LOGGER.warn(e)
            problem_passed = False

        if problem_passed:
            jacobian_cls = create_jacobian(options)
            jacobian = jacobian_cls(parsed_problem)

            # Making the Jacobian class part of the fitting problem. This will
            # eventually be extended to have Hessian information too.
            parsed_problem.jac = jacobian
            ##############################
            # Loops over starting values #
            ##############################
            problem_results, problem_fails, \
                unselected_minimzers = loop_over_starting_values(
                    parsed_problem, options, grabbed_output)
            results.extend(problem_results)
            failed_problems.extend(problem_fails)
    # If the results are an empty list then this means that all minimizers
    # raise an exception and the tables will produce errors if they run.
    if results == []:
        message = "The user chosen options and/or problem setup resulted in " \
            "all minimizers and/or parsers raising an exception. " \
            "This is likely due to the way `algorithm_type` was set " \
                  "in the options or the selected problem set requires " \
                  "additional software to be installed. Please review your " \
                  "options setup and/or problem set then re-run " \
                  "FitBenmarking."
        raise NoResultsError(message)

    return results, failed_problems, unselected_minimzers


def loop_over_starting_values(problem, options, grabbed_output):
    """
    Loops over starting values from the fitting problem

    :param problem: a problem object containing information used in fitting
    :type problem: FittingProblem
    :param options: FitBenchmarking options for current run
    :type options: fitbenchmarking.utils.options.Options
    :param grabbed_output: Object that removes third part output from console
    :type grabbed_output: fitbenchmarking.utils.output_grabber.OutputGrabber

    :return: prob_results array of fitting results for the problem group,
             list of failed problems and dictionary of unselected minimizers
    :rtype: tuple(list, list, dict)
    """
    name = problem.name
    num_start_vals = len(problem.starting_values)
    problem_results = []
    for index in range(num_start_vals):
        LOGGER.info("    Starting value: {0}/{1}".format(index + 1,
                                                         num_start_vals))
        if num_start_vals > 1:
            problem.name = name + ', Start {}'.format(index + 1)

        ################################
        # Loops over fitting softwares #
        ################################
        individual_problem_results, problem_fails, unselected_minimzers = loop_over_fitting_software(
            problem=problem, options=options,
            start_values_index=index,
            grabbed_output=grabbed_output)
        problem_results.extend(individual_problem_results)

    return problem_results, problem_fails, unselected_minimzers


def loop_over_fitting_software(problem, options, start_values_index,
                               grabbed_output):
    """
    Loops over fitting software selected in the options

    :param problem: a problem object containing information used in fitting
    :type problem: FittingProblem
    :param options: FitBenchmarking options for current run
    :type options: fitbenchmarking.utils.options.Options
    :param start_values_index: Integer that selects the starting values when
                               datasets have multiple ones.
    :type start_values_index: int
    :param grabbed_output: Object that removes third part output from console
    :type grabbed_output: fitbenchmarking.utils.output_grabber.OutputGrabber

    :return: list of all results, failed problem names and dictionary of
             unselected minimizers based on algorithm_type
    :rtype: tuple(list of fibenchmarking.utils.fitbm_result.FittingResult,
                  list of failed problem names,
                  dictionary of minimizers)
    """
    results = []
    software = options.software
    if not isinstance(software, list):
        software = [software]

    problem_fails = []
    unselected_minimzers = {}
    software_results = []
    for s in software:
        LOGGER.info("        Software: %s", s.upper())
        try:
            minimizers = options.minimizers[s]
        except KeyError:
            raise UnsupportedMinimizerError(
                'No minimizer given for software: {}'.format(s))
        with grabbed_output:
            controller_cls = ControllerFactory.create_controller(
                software=s)
            controller = controller_cls(problem=problem)

        controller.parameter_set = start_values_index

        #########################
        # Loops over minimizers #
        #########################
        problem_result, minimizer_failed = loop_over_minimizers(controller=controller,
                                                                minimizers=minimizers,
                                                                options=options,
                                                                grabbed_output=grabbed_output)
        unselected_minimzers[s] = minimizer_failed
        software_results.extend(problem_result)

    # Checks to see if all of the minimizers raised an exception and
    # record the problem name if that is the case
    software_check = [np.isinf(v.chi_sq) for v in software_results]
    if all(software_check):
        software_results = []
        problem_fails.append(problem.name)
    results.extend(software_results)

    return results, problem_fails, unselected_minimzers


def loop_over_minimizers(controller, minimizers, options, grabbed_output):
    """
    Loops over minimizers in fitting software

    :param controller: The software controller for the fitting
    :type controller: Object derived from BaseSoftwareController
    :param minimizers: array of minimizers used in fitting
    :type minimizers: list
    :param options: FitBenchmarking options for current run
    :type options: fitbenchmarking.utils.options.Options
    :param grabbed_output: Object that removes third part output from console
    :type grabbed_output: fitbenchmarking.utils.output_grabber.OutputGrabber

    :return: list of all results and dictionary of unselected minimizers
             based on algorithm_type
    :rtype: tuple(list of fibenchmarking.utils.fitbm_result.FittingResult,
                  list of failed minimizers)
    """
    grabbed_output = output_grabber.OutputGrabber(options)
    problem = controller.problem
    jac = controller.problem.jac

    results_problem = []
    minimizer_failed = []

    num_runs = options.num_runs
    algorithm_type = options.algorithm_type

    for minimizer in minimizers:
        minimizer_check = True
        LOGGER.info("            Minimizer: %s", minimizer)

        controller.minimizer = minimizer
        try:
            with grabbed_output:
                controller.validate_minimizer(minimizer, algorithm_type)
                # Calls timeit repeat with repeat = num_runs and number = 1
                runtime_list = timeit.Timer(setup=controller.prepare,
                                            stmt=controller.fit).repeat(num_runs, 1)
                runtime = sum(runtime_list) / num_runs
                controller.cleanup()
                controller.check_attributes()
            ratio = np.max(runtime_list) / np.min(runtime_list)
            tol = 4
            if ratio > tol:
                warnings.warn('The ratio of the max time to the min is {0}'
                              ' which is  larger than the tolerance of {1},'
                              ' which may indicate that caching has occurred'
                              ' in the timing results'.format(ratio, tol))
            chi_sq = controller.eval_chisq(params=controller.final_params,
                                           x=controller.data_x,
                                           y=controller.data_y,
                                           e=controller.data_e)
            chi_sq_check = any(np.isnan(n) for n in chi_sq) \
                if problem.multifit else np.isnan(chi_sq)
            if np.isnan(runtime) or chi_sq_check:
                raise ControllerAttributeError(
                    "Either the computed runtime or chi_sq values "
                    "was a NaN.")
        # Catching all exceptions as this means runtime cannot be calculated
        # pylint: disable=broad-except
        except Exception as excp:
            if isinstance(excp, UnknownMinimizerError):
                minimizer_failed.append(minimizer)
                minimizer_check = False
            LOGGER.warn(str(excp))

            runtime = np.inf
            controller.flag = 3
            controller.final_params = None if not problem.multifit \
                else [None] * len(controller.data_x)
            chi_sq = np.inf if not problem.multifit \
                else [np.inf] * len(controller.data_x)

        result_args = {'options': options,
                       'problem': problem,
                       'jac': jac,
                       'chi_sq': chi_sq,
                       'runtime': runtime,
                       'minimizer': minimizer,
                       'initial_params': controller.initial_params,
                       'params': controller.final_params,
                       'error_flag': controller.flag,
                       'name': problem.name}

        if minimizer_check:
            if problem.multifit:
                # Multi fit (will raise TypeError if these are not iterable)
                for i in range(len(chi_sq)):
                    result_args.update({'dataset_id': i,
                                        'name': '{}, Dataset {}'.format(
                                            problem.name, (i + 1))})
                    individual_result = fitbm_result.FittingResult(
                        **result_args)
                    results_problem.append(individual_result)
            else:
                # Normal fitting
                individual_result = fitbm_result.FittingResult(**result_args)
                results_problem.append(individual_result)
    return results_problem, minimizer_failed



"""
Results output tests
"""

from __future__ import (absolute_import, division, print_function)
import unittest
import os
import shutil
import numpy as np

from fitbenchmarking.jacobian.SciPyFD_2point_jacobian import ScipyTwoPoint
from fitbenchmarking.parsing.fitting_problem import FittingProblem
from fitbenchmarking.utils.fitbm_result import FittingResult
from fitbenchmarking.core.results_output import create_directories, \
    save_results, preproccess_data, create_plots, create_problem_level_index
from fitbenchmarking.utils.options import Options


# By design both fitting_function_1 and fitting_function_2 need data as an
# argument
# pylint: disable=unused-argument
def fitting_function_1(data, x1, x2):
    """
    Fitting function evaluator

    :param data: x data
    :type data: numpy array
    :param x1: fitting parameter
    :type x1: float
    :param x2: fitting parameter
    :type x2: float

    :return: y data values evaluated from the function of the problem
    :rtype: numpy array
    """
    return x1 * np.sin(x2)


def fitting_function_2(data, x1, x2):
    """
    Fitting function evaluator

    :param data: x data
    :type data: numpy array
    :param x1: fitting parameter
    :type x1: float
    :param x2: fitting parameter
    :type x2: float

    :return: y data values evaluated from the function of the problem
    :rtype: numpy array
    """
    return x1 * x2


# pylint: enable=unused-argument
def generate_mock_results():
    """
    Generates results to test against

    :return: best results calculated using the chi_sq value, list of results
             and the options
    :rtype: tuple(list of best results,
                  list of list fitting results,
                  Options object)
    """
    software = 'scipy_ls'
    options = Options()
    options.software = [software]
    num_min = len(options.minimizers[options.software[0]])
    data_x = np.array([[1, 4, 5], [2, 1, 5]])
    data_y = np.array([[1, 2, 1], [2, 2, 2]])
    data_e = np.array([[1, 1, 1], [1, 2, 1]])
    func = [fitting_function_1, fitting_function_2]
    problems = [FittingProblem(options), FittingProblem(options)]

    params_in = [[[.3, .11], [.04, 2], [3, 1], [5, 0]],
                 [[4, 2], [3, .006], [.3, 10], [9, 0]]]

    starting_values = [{"a": .3, "b": .11}, {"a": 0, "b": 0}]
    error_in = [[1, 0, 2, 0],
                [0, 1, 3, 1]]
    link_in = [['link1', 'link2', 'link3', 'link4'],
               ['link5', 'link6', 'link7', 'link8']]
    min_chi_sq = [1, 1]
    acc_in = [[1, 5, 2, 1.54],
              [7, 3, 5, 1]]
    min_runtime = [4.2e-5, 5.0e-14]
    runtime_in = [[1e-2, 2.2e-3, 4.2e-5, 9.8e-1],
                  [3.0e-10, 5.0e-14, 1e-7, 4.3e-12]]

    results_out = []
    for i, p in enumerate(problems):
        p.data_x = data_x[i]
        p.data_y = data_y[i]
        p.data_e = data_e[i]
        p.function = func[i]
        p.name = "prob_{}".format(i)
        results = []
        for j in range(num_min):
            p.starting_values = starting_values
            jac = ScipyTwoPoint(p)
            jac.method = '2-point'
            r = FittingResult(options=options, problem=p, jac=jac,
                              initial_params=starting_values,
                              params=params_in[i][j])
            r.chi_sq = acc_in[i][j]
            r.runtime = runtime_in[i][j]
            r.error_flag = error_in[i][j]
            r.support_page_link = link_in[i][j]
            r.minimizer = options.minimizers[software][j]
            results.append(r)
        results_out.append(results)
    return results_out, options, min_chi_sq, min_runtime


class SaveResultsTests(unittest.TestCase):
    """
    Unit tests for save_results function
    """

    def setUp(self):
        """
        Setting up paths and results folders
        """
        self.results, self.options, self.min_chi_sq, self.min_runtime = \
            generate_mock_results()
        test_path = os.path.dirname(os.path.realpath(__file__))
        self.dirname = os.path.join(test_path, 'fitbenchmarking_results')
        self.options.results_dir = self.dirname
        os.mkdir(self.dirname)

    def tearDown(self):
        """
        Clean up created folders.
        """
        shutil.rmtree(self.dirname)

    def test_save_results_correct(self):
        """
        Tests to check the group_dir is correct
        """
        failed_problems = []
        unselected_minimzers = {}
        group_name = "group_name"
        group_dir = save_results(self.options, self.results, group_name,
                                 failed_problems, unselected_minimzers)
        assert group_dir == os.path.join(self.dirname, group_name)


class CreateDirectoriesTests(unittest.TestCase):
    """
    Unit tests for create_directories function
    """

    def setUp(self):
        """
        Setting up paths and results folders
        """
        self.options = Options()
        test_path = os.path.dirname(os.path.realpath(__file__))
        self.dirname = os.path.join(test_path, 'fitbenchmarking_results')
        os.mkdir(self.dirname)

    def tearDown(self):
        """
        Clean up created folders.
        """
        shutil.rmtree(self.dirname)

    def test_create_dirs_correct(self):
        """
        Test to check that the directories are correctly made.
        """
        group_name = 'test_group'
        self.options.results_dir = self.dirname
        expected_results_dir = self.dirname
        expected_group_dir = os.path.join(expected_results_dir, group_name)
        expected_support_dir = os.path.join(expected_group_dir,
                                            'support_pages')
        expected_figures_dir = os.path.join(expected_support_dir, 'figures')
        expected_css_dir = os.path.join(expected_results_dir,'css')

        group_dir, support_dir, figures_dir, css_dir = \
            create_directories(self.options, group_name)

        assert group_dir == expected_group_dir
        assert support_dir == expected_support_dir
        assert figures_dir == expected_figures_dir
        assert css_dir == expected_css_dir

        assert os.path.isdir(group_dir)
        assert os.path.isdir(support_dir)
        assert os.path.isdir(figures_dir)
        assert os.path.isdir(css_dir)


class PreproccessDataTests(unittest.TestCase):
    """
    Unit tests for preproccess_data function
    """

    def setUp(self):
        """
        Setting up paths and results folders
        """
        self.results, self.options, self.min_chi_sq, self.min_runtime = \
            generate_mock_results()

    def test_preproccess_data(self):
        """
        Test for preproccess_data function
        """
        best_result = preproccess_data(self.results)

        for result in best_result:
            assert result.is_best_fit

        for result, chi_sq, runtime in zip(self.results,
                                           self.min_chi_sq,
                                           self.min_runtime):
            assert all(r.min_chi_sq == chi_sq for r in result)
            assert all(r.min_runtime == runtime for r in result)


class CreatePlotsTests(unittest.TestCase):
    """
    Unit tests for create_plots function
    """

    def setUp(self):
        """
        Setting up paths and results folders
        """
        self.results, self.options, self.min_chi_sq, self.min_runtime = \
            generate_mock_results()
        self.best_results = preproccess_data(self.results)

    @unittest.mock.patch('fitbenchmarking.results_processing.plots.Plot')
    def test_create_plots_with_params(self, plot_mock):
        """
        Tests for create_plots where the results object params are not None
        """
        expected_plot_initial_guess = "initial_guess"
        expected_plot_best = "plot_best"
        expected_plot_fit = "plot_fit"
        plot_instance = unittest.mock.MagicMock()
        plot_instance.plot_initial_guess.return_value = \
            expected_plot_initial_guess
        plot_instance.plot_best.return_value = expected_plot_best
        plot_instance.plot_fit.return_value = expected_plot_fit

        # Return the above created `plot_instance`
        plot_mock.return_value = plot_instance
        create_plots(self.options, self.results,
                     self.best_results, "figures_dir")
        for result, best_result in zip(self.results, self.best_results):
            # Check initial guess is correctly set in results
            assert best_result.start_figure_link == expected_plot_initial_guess
            assert all(r.start_figure_link == expected_plot_initial_guess
                       for r in result)

            # Check plot is correctly set in results
            assert best_result.figure_link == expected_plot_best
            assert all(r.figure_link == expected_plot_fit
                       if not r.is_best_fit else True for r in result)

    @unittest.mock.patch('fitbenchmarking.results_processing.plots.Plot')
    def test_create_plots_without_params(self, plot_mock):
        """
        Tests for create_plots where the results object params are None
        """
        for result, best in zip(self.results, self.best_results):
            best.params = None
            for r in result:
                r.params = None
        expected_plot_initial_guess = "initial_guess"
        plot_instance = unittest.mock.MagicMock()
        plot_instance.plot_initial_guess.return_value = \
            expected_plot_initial_guess

        # Return the above created `plot_instance`
        plot_mock.return_value = plot_instance
        create_plots(self.options, self.results,
                     self.best_results, "figures_dir")
        for result, best_result in zip(self.results, self.best_results):
            # Check initial guess is correctly set in results
            assert best_result.start_figure_link == expected_plot_initial_guess
            assert all(r.start_figure_link == expected_plot_initial_guess
                       for r in result)

            # Check plot is correctly set in results
            assert best_result.figure_link == ''
            assert all(r.figure_link == ''
                       if not r.is_best_fit else True for r in result)

            # Checks that when no params are given the correct error message
            # is produced
            expected_message = "Minimizer failed to produce any parameters"
            assert best_result.figure_error == expected_message
            assert all(r.figure_error == expected_message
                       if not r.is_best_fit else True for r in result)


class CreateProblemLevelIndex(unittest.TestCase):
    """
   Unit tests for create_problem_level_index function
   """

    def setUp(self):
        """
        Setting up paths and results folders
        """
        self.options = Options()
        test_path = os.path.dirname(os.path.realpath(__file__))
        self.group_dir = os.path.join(test_path, 'fitbenchmarking_results')
        os.mkdir(self.group_dir)
        self.table_names = {"compare": "compare_table_name.",
                            "runtime": "runtime_table_name."}
        self.table_descriptions = {"compare": "compare table descriptions",
                                   "runtime": "runtime table descriptions",
                                   "both": "both table descriptions"}
        self.group_name = "random_name"

    def tearDown(self):
        """
        Clean up created folders.
        """
        shutil.rmtree(self.group_dir)

    def test_creation_index_page(self):
        """
        Tests to see that the index page are correctly created
        """
        create_problem_level_index(self.options, self.table_names,
                                   self.group_name, self.group_dir,
                                   self.table_descriptions)
        expected_file = os.path.join(self.group_dir,
                                     '{}_index.html'.format(self.group_name))
        assert os.path.isfile(expected_file)


if __name__ == "__main__":
    unittest.main()



"""
Implements a controller for the Mantid fitting software.
"""

from mantid import simpleapi as msapi
from mantid.fitfunctions import FunctionFactory, FunctionWrapper, IFunction1D
import numpy as np

from fitbenchmarking.controllers.base_controller import Controller


class MantidController(Controller):
    """
    Controller for the Mantid fitting software.

    Mantid requires subscribing a custom function in a predefined format,
    so this controller creates that in setup.
    """

    def __init__(self, problem):
        """
        Setup workspace, cost_function, ignore_invalid, and initialise vars
        used for temporary storage within the mantid controller

        :param problem: Problem to fit
        :type problem: FittingProblem
        """
        super(MantidController, self).__init__(problem)

        self._param_names = self.problem.param_names
        self.algorithm_check = {
            'all': ['BFGS', 'Conjugate gradient (Fletcher-Reeves imp.)',
                    'Conjugate gradient (Polak-Ribiere imp.)',
                    'Damped GaussNewton', 'Levenberg-Marquardt',
                    'Levenberg-MarquardtMD', 'Simplex', 'SteepestDescent',
                    'Trust Region'],
            'ls': ['Levenberg-Marquardt', 'Levenberg-MarquardtMD',
                   'Trust Region'],
            'deriv_free': ['Simplex'],
            'general': ['BFGS', 'Conjugate gradient (Fletcher-Reeves imp.)',
                        'Conjugate gradient (Polak-Ribiere imp.)',
                        'Damped GaussNewton', 'Simplex', 'SteepestDescent']}

        if self.problem.multifit:
            # Multi Fit
            use_errors = self.data_e[0] is not None
            data_obj = msapi.CreateWorkspace(DataX=self.data_x[0],
                                             DataY=self.data_y[0],
                                             DataE=self.data_e[0],
                                             OutputWorkspace='ws0')
            other_inputs = [
                msapi.CreateWorkspace(DataX=x, DataY=y, DataE=e,
                                      OutputWorkspace='ws{}'.format(i + 1))
                for i, (x, y, e) in enumerate(zip(self.data_x[1:],
                                                  self.data_y[1:],
                                                  self.data_e[1:]))]
            self._multi_fit = len(other_inputs) + 1
        else:
            # Normal Fitting
            use_errors = self.data_e is not None
            data_obj = msapi.CreateWorkspace(DataX=self.data_x,
                                             DataY=self.data_y,
                                             DataE=self.data_e)
            other_inputs = []
            self._multi_fit = 0

        self._cost_function = 'Least squares' if use_errors \
            else 'Unweighted least squares'
        self._mantid_data = data_obj
        self._mantid_function = None
        self._mantid_results = None

        # Use the raw string format if this is from a Mantid problem.
        # This enables advanced features such as contraints.
        try:
            function_def = self.problem.additional_info['mantid_equation']
            if self._multi_fit:
                # Each function must include '$domains=i'
                if ';' in function_def:
                    function_def = ' (composite=CompositeFunction, '\
                                   + 'NumDeriv=false, $domains=i; '\
                                   + '{});'.format(function_def)
                else:
                    function_def += ', $domains=i; '

                # Multi fit must have 'composite=MultiDomainFunction' in the
                # first function.
                composite_str = 'composite=MultiDomainFunction, NumDeriv=1;'
                function_def = composite_str + function_def * self._multi_fit
                ties = ','.join('f{0}.{1}=f0.{1}'.format(i, p)
                                for p in self.problem.additional_info[
                                    'mantid_ties']
                                for i in range(1, self._multi_fit))
                function_def += 'ties=({})'.format(ties)

            self._mantid_function = function_def
        except KeyError:
            # This will be completed in setup as it requires initial params
            self._mantid_function = None

        # Arguments will change if multi-data
        self._added_args = {'InputWorkspace_{}'.format(i + 1): v
                            for i, v in enumerate(other_inputs)}

    def setup(self):
        """
        Setup problem ready to run with Mantid.

        Adds a custom function to Mantid for calling in fit().
        """

        if self._mantid_function is None:
            start_val_list = ['{0}={1}'.format(name, value)
                              for name, value
                              in zip(self._param_names, self.initial_params)]

            start_val_str = ', '.join(start_val_list)
            function_def = "name=fitFunction, " + start_val_str

            class fitFunction(IFunction1D):
                def init(ff_self):

                    for param in self._param_names:
                        ff_self.declareParameter(param)

                def function1D(ff_self, xdata):

                    fit_param = np.zeros(len(self._param_names))
                    fit_param.setflags(write=1)
                    for i, param in enumerate(self._param_names):
                        fit_param[i] = ff_self.getParameterValue(param)

                    return self.problem.eval_f(x=xdata,
                                               params=fit_param)

            FunctionFactory.subscribe(fitFunction)

            self._mantid_function = function_def

    def fit(self):
        """
        Run problem with Mantid.
        """
        fit_result = msapi.Fit(Function=self._mantid_function,
                               CostFunction=self._cost_function,
                               Minimizer=self.minimizer,
                               InputWorkspace=self._mantid_data,
                               Output='fit',
                               **self._added_args)

        self._mantid_results = fit_result
        self._status = self._mantid_results.OutputStatus

    def cleanup(self):
        """
        Convert the result to a numpy array and populate the variables results
        will be read from.
        """
        if self._status == "success":
            self.flag = 0
        elif "Failed to converge" in self._status:
            self.flag = 1
        else:
            self.flag = 2

        # Mantid gives chi sq as last elem in params
        final_params = self._mantid_results.OutputParameters.column(1)[:-1]
        num_params = len(self.initial_params)
        if self._multi_fit:
            self.final_params = [final_params[i * num_params:(i + 1) * num_params]
                                 for i in range(self._multi_fit)]
        else:
            self.final_params = final_params

    # Override if multi-fit
    # =====================
    def eval_chisq(self, params, x=None, y=None, e=None):
        """
        Computes the chisq value.
        If multi-fit inputs will be lists and this will return a list of chi
        squared of params[i], x[i], y[i], and e[i].

        :param params: The parameters to calculate residuals for
        :type params: list of float or list of list of float
        :param x: x data points, defaults to self.data_x
        :type x: numpy array or list of numpy arrays, optional
        :param y: y data points, defaults to self.data_y
        :type y: numpy array or list of numpy arrays, optional
        :param e: error at each data point, defaults to self.data_e
        :type e: numpy array or list of numpy arrays, optional

        :return: The sum of squares of residuals for the datapoints at the
                 given parameters
        :rtype: numpy array
        """
        if x is not None:
            # If x[0] is scalar this is false, otherwise x is a list of
            # datasets
            multifit = bool(np.shape(x[0]))
        else:
            multifit = self._multi_fit

        if multifit:
            num_inps = len(params)
            if x is None:
                x = [None for _ in range(num_inps)]
            if y is None:
                y = [None for _ in range(num_inps)]
            if e is None:
                e = [None for _ in range(num_inps)]
            return [super(MantidController, self).eval_chisq(p, xi, yi, ei)
                    for p, xi, yi, ei in zip(params, x, y, e)]
        else:
            return super(MantidController, self).eval_chisq(params, x, y, e)



'''
Test the FITTING section for the options file
'''
import inspect
import shutil
import os
import unittest

from fitbenchmarking.utils import exceptions
from fitbenchmarking.utils.options import Options


class FittingOptionTests(unittest.TestCase):
    """
    Checks that the defaults in the FITTING section are set correctly
    """

    def setUp(self):
        """
        Initializes options class with defaults
        """
        self.options = Options()

    def test_num_runs_default(self):
        """
        Checks num_runs default
        """
        expected = 5
        actual = self.options.num_runs
        self.assertEqual(expected, actual)

    def test_algorithm_type_default(self):
        """
        Checks algorithm_type default
        """
        expected = 'all'
        actual = self.options.algorithm_type
        self.assertEqual(expected, actual)

    def test_software_default(self):
        """
        Checks software default
        """
        expected = ['scipy', 'scipy_ls']
        actual = self.options.software
        self.assertEqual(expected, actual)

    def test_use_errors_default(self):
        """
        Checks use_errors default
        """
        expected = True
        actual = self.options.use_errors
        self.assertEqual(expected, actual)

    def test_jac_method_default(self):
        """
        Checks jac_method default
        """
        expected = 'SciPyFD'
        actual = self.options.jac_method
        self.assertEqual(expected, actual)

    def test_num_method_default(self):
        """
        Checks num_method default
        """
        expected = '2point'
        actual = self.options.num_method
        self.assertEqual(expected, actual)


class UserFittingOptionTests(unittest.TestCase):
    """
    Checks the fitting in the options file are set correctly or raise errors
    """

    def setUp(self):
        """
        Sets the directory to save the temporary ini files in
        """
        options_dir = os.path.dirname(inspect.getfile(Options))
        self.test_files_dir = os.path.join(options_dir, 'tests', 'files')
        os.mkdir(self.test_files_dir)

    def tearDown(self):
        """
        Deletes temporary folder and results produced
        """
        if os.path.exists(self.test_files_dir):
            shutil.rmtree(self.test_files_dir)

    def generate_user_ini_file(self, opt_name, config_str):
        """
        Generates user defined ini file

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: section of an ini file which sets the option
        :type config_str: str

        :return: location of temporary ini file
        :rtype: str
        """
        opts_file = os.path.join(self.test_files_dir,
                                 'test_{}_valid.ini'.format(opt_name))
        with open(opts_file, 'w') as f:
            f.write(config_str)
        return opts_file

    def shared_valid(self, opt_name, options_set, config_str):
        """
        Shared test to check that the fitting option set is valid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param options_set: option set to be tested
        :type options_set: list
        :param config_str: section of an ini file which sets the option
        :type config_str: str
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        options = Options(opts_file)
        actual = getattr(options, opt_name)
        self.assertEqual(options_set, actual)

    def shared_invalid(self, opt_name, config_str):
        """
        Shared test to check that the fitting option set is invalid

        :param opt_name: name of option to be set
        :type opt_name: str
        :param config_str: option set to be tested
        :type config_str: list
        """
        opts_file = self.generate_user_ini_file(opt_name, config_str)
        with self.assertRaises(exceptions.OptionsError):
            Options(opts_file)

    def test_invalid_option_key(self):
        """
        Tests that the user defined option key is invalid.
        """
        config_str = \
            "[FITTING]\nnum_minimizer_runs: 3"
        self.shared_invalid('num_minimizer_runs', config_str)

    def test_minimizer_num_runs_valid(self):
        """
        Checks user set num_runs is valid
        """
        set_option = 3
        config_str = \
            "[FITTING]\nnum_runs: 3"
        self.shared_valid('num_runs', set_option, config_str)

    def test_minimizer_num_runs_invalid(self):
        """
        Checks user set num_runs is invalid
        """
        config_str = \
            "[FITTING]\nnum_runs: two"
        self.shared_invalid('num_runs', config_str)

    def test_minimizer_algorithm_type_valid(self):
        """
        Checks user set algorithm_type is valid
        """
        set_option = 'general'
        config_str = \
            "[FITTING]\nalgorithm_type: general"
        self.shared_valid('algorithm_type', set_option, config_str)

    def test_minimizer_algorithm_type_invalid(self):
        """
        Checks user set algorithm_type is invalid
        """
        config_str = \
            "[FITTING]\nalgorithm_type: all_the_minimizers"
        self.shared_invalid('algorithm_type', config_str)

    def test_minimizer_use_errors_valid(self):
        """
        Checks user set use_errors is valid
        """
        set_option = False
        config_str = \
            "[FITTING]\nuse_errors: no"
        self.shared_valid('use_errors', set_option, config_str)

    def test_minimizer_use_errors_invalid(self):
        """
        Checks user set use_errors is invalid
        """
        config_str = \
            "[FITTING]\nuse_errors: use_weights"
        self.shared_invalid('use_errors', config_str)

    def test_minimizer_jac_method_valid(self):
        """
        Checks user set jac_method is valid
        """
        set_option = "SciPyFD"
        config_str = \
            "[FITTING]\njac_method: SciPyFD"
        self.shared_valid('jac_method', set_option, config_str)

    def test_minimizer_jac_method_invalid(self):
        """
        Checks user set jac_method is invalid
        """
        config_str = \
            "[FITTING]\njac_method: NumPyFD"
        self.shared_invalid('jac_method', config_str)

    def test_minimizer_num_method_valid(self):
        """
        Checks user set num_method is valid
        """
        set_option = "cs"
        config_str = \
            "[FITTING]\nnum_method: cs"
        self.shared_valid('num_method', set_option, config_str)

    def test_minimizer_num_method_invalid(self):
        """
        Checks user set num_method is invalid
        """
        config_str = \
            "[FITTING]\nnum_method: FD_3point"
        self.shared_invalid('num_method', config_str)



"""
Tests for fitbenchmarking.core.fitting_benchmarking.loop_over_starting_values
"""
from __future__ import (absolute_import, division, print_function)
import inspect
import os
import unittest

from fitbenchmarking import mock_problems
from fitbenchmarking.utils import fitbm_result, output_grabber
from fitbenchmarking.core.fitting_benchmarking import loop_over_starting_values
from fitbenchmarking.parsing.parser_factory import parse_problem_file
from fitbenchmarking.utils.options import Options
from fitbenchmarking.jacobian.SciPyFD_2point_jacobian import ScipyTwoPoint

# Defines the module which we mock out certain function calls for
FITTING_DIR = "fitbenchmarking.core.fitting_benchmarking"


# Due to structure of tests, some variables may not be previously defined
# in the init function. Removes pytest dictionary iteration suggestion
# pylint: disable=attribute-defined-outside-init, consider-iterating-dictionary
def make_fitting_problem(file_name='cubic.dat', minimizers=None):
    """
    Helper function that returns a simple fitting problem
    """
    options = Options()
    if minimizers:
        options.minimizers = minimizers

    bench_prob_dir = os.path.dirname(inspect.getfile(mock_problems))
    fname = os.path.join(bench_prob_dir, file_name)

    fitting_problem = parse_problem_file(fname, options)
    fitting_problem.correct_data()
    jac = ScipyTwoPoint(fitting_problem)
    fitting_problem.jac = jac
    return fitting_problem


class LoopOverStartingValuesTests(unittest.TestCase):
    """
    loop_over_starting_values tests
    """

    def setUp(self):
        """
        Setting up problem for tests
        """
        self.problem = make_fitting_problem()
        self.options = self.problem.options
        self.options.software = ["scipy"]
        self.minimizers = self.options.minimizers
        self.grabbed_output = output_grabber.OutputGrabber(self.options)
        self.count = 0
        self.scipy_len = len(self.options.minimizers["scipy"])
        self.result_args = {'options': self.options,
                            'problem': self.problem,
                            'jac': self.problem.jac,
                            'initial_params': self.problem.starting_values[0],
                            'params': [],
                            'chi_sq': 1}

    def mock_func_call(self, *args, **kwargs):
        """
        Mock function to be used instead of loop_over_fitting_software
        """
        individual_problem_results = \
            self.individual_problem_results[self.count]
        problem_fails = self.problem_fails
        unselected_minimzers = self.unselected_minimzers
        self.count += 1
        return individual_problem_results, problem_fails, unselected_minimzers

    def shared_tests(self, expected_list_len, expected_problem_fails,
                     expected_unselected_minimzers):
        """
        Shared tests for the `loop_over_starting_values` function

        :param expected_list_len: number of expect fitting results
        :type expected_list_len: int
        :param expected_problem_fails: Expected list of failed problems
        :type expected_problem_fails: list
        :param expected_unselected_minimzers: Expected dictionary of unselected
                                              minimizer
        :type expected_unselected_minimzers: dict
        """
        problem_results, problem_fails, unselected_minimzers = \
            loop_over_starting_values(self.problem,
                                      self.options,
                                      self.grabbed_output)
        assert len(problem_results) == expected_list_len
        assert problem_fails == expected_problem_fails

        for key in unselected_minimzers.keys():
            assert key in expected_unselected_minimzers.keys()
            assert sorted(unselected_minimzers[key]) == \
                sorted(expected_unselected_minimzers[key])

    @unittest.mock.patch('{}.loop_over_fitting_software'.format(FITTING_DIR))
    def test_run_multiple_starting_values(self, loop_over_fitting_software):
        """
        Checks that all selected minimizers run with multiple starting
        values
        """
        list_results = [fitbm_result.FittingResult(**self.result_args)
                        for i in range(self.scipy_len)]
        self.individual_problem_results = [list_results, list_results]
        self.problem_fails = []
        self.unselected_minimzers = {"scipy": []}
        loop_over_fitting_software.side_effect = self.mock_func_call
        expected_list_length = len(list_results) * 2
        expected_problem_fails = self.problem_fails
        expected_unselected_minimzers = self.unselected_minimzers
        self.shared_tests(expected_list_length, expected_problem_fails,
                          expected_unselected_minimzers)

    @unittest.mock.patch('{}.loop_over_fitting_software'.format(FITTING_DIR))
    def test_run_one_starting_values(self, loop_over_fitting_software):
        """
        Checks that all selected minimizers run with one starting
        values
        """
        self.problem.starting_values = [self.problem.starting_values[0]]
        self.individual_problem_results = \
            [[fitbm_result.FittingResult(**self.result_args)
              for i in range(self.scipy_len)]]
        self.problem_fails = []
        self.unselected_minimzers = {"scipy": []}
        loop_over_fitting_software.side_effect = self.mock_func_call
        expected_list_length = len(self.individual_problem_results[0])
        expected_problem_fails = self.problem_fails
        expected_unselected_minimzers = self.unselected_minimzers
        self.shared_tests(expected_list_length, expected_problem_fails,
                          expected_unselected_minimzers)

    @unittest.mock.patch('{}.loop_over_fitting_software'.format(FITTING_DIR))
    def test_run_reports_failed_problems(self, loop_over_fitting_software):
        """
        Checks that the failed problems are reported correctly
        """
        list_results = [fitbm_result.FittingResult(**self.result_args)
                        for i in range(self.scipy_len)]
        self.individual_problem_results = [list_results, list_results]
        self.problem_fails = ['Cubic_failed']
        self.unselected_minimzers = {"scipy": []}
        loop_over_fitting_software.side_effect = self.mock_func_call
        expected_list_length = len(list_results) * 2
        expected_problem_fails = self.problem_fails
        expected_unselected_minimzers = self.unselected_minimzers
        self.shared_tests(expected_list_length, expected_problem_fails,
                          expected_unselected_minimzers)

    @unittest.mock.patch('{}.loop_over_fitting_software'.format(FITTING_DIR))
    def test_run_reports_unselected_minimizers(self,
                                               loop_over_fitting_software):
        """
        Checks that the unselected minimizers are reported correctly
        """
        list_results = [fitbm_result.FittingResult(**self.result_args)
                        for i in range(self.scipy_len)]
        self.individual_problem_results = [list_results, list_results]
        self.problem_fails = []
        self.unselected_minimzers = {"scipy": ['Powell', 'CG']}
        loop_over_fitting_software.side_effect = self.mock_func_call
        expected_list_length = len(list_results) * 2
        expected_problem_fails = self.problem_fails
        expected_unselected_minimzers = self.unselected_minimzers
        self.shared_tests(expected_list_length, expected_problem_fails,
                          expected_unselected_minimzers)


if __name__ == "__main__":
    unittest.main()



"""
Test that accuracy of FitBenchmarking is consistent with previous versions
"""

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest
import os
from sys import platform
import tempfile
from unittest import TestCase
import pytest
from pytest import test_type as TEST_TYPE

from fitbenchmarking.cli.main import run
from fitbenchmarking.utils.options import Options


@pytest.mark.skipif("TEST_TYPE == 'default'")
class TestRegressionAll(TestCase):
    """
    Regression tests for the Fitbenchmarking software with all fitting software
    packages
    """

    @classmethod
    def setUpClass(cls):
        """
        Create an options file, run it, and get the results.
        """
        opts = setup_options()
        opt_file = tempfile.NamedTemporaryFile(suffix='.ini',
                                               mode='w',
                                               delete=False)
        opts.write_to_stream(opt_file)
        opt_file.close()
        problem = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                               os.pardir,
                                               'mock_problems',
                                               'all_parsers_set'))
        run([problem], options_file=opt_file.name, debug=True)
        os.remove(opt_file.name)
        opts = setup_options(multifit=True)
        opt_file = tempfile.NamedTemporaryFile(suffix='.ini',
                                               mode='w',
                                               delete=False)
        opts.write_to_stream(opt_file)
        opt_file.close()
        problem = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                               os.pardir,
                                               'mock_problems',
                                               'multifit_set'))
        run([problem], options_file=opt_file.name, debug=True)
        os.remove(opt_file.name)

    def test_results_consistent_all(self):
        """
        Regression testing that the results of fitting a set of problems
        containing all problem types against a single minimizer from each of
        the supported softwares
        """

        expected_file = os.path.join(os.path.dirname(__file__),
                                     '{}_expected_results'.format(platform),
                                     'all_parsers.txt')

        actual_file = os.path.join(os.path.dirname(__file__),
                                   'fitbenchmarking_results',
                                   'all_parsers_set',
                                   'all_parsers_set_acc_weighted_table.txt')

        with open(expected_file, 'r') as f:
            expected = f.readlines()

        with open(actual_file, 'r') as f:
            actual = f.readlines()

        diff, msg = diff_result(actual, expected)
        self.assertListEqual([], diff, msg)

    def test_multifit_consistent(self):
        """
        Regression testing that the results of fitting multifit problems
        against a single minimizer from mantid.
        """

        expected_file = os.path.join(os.path.dirname(__file__),
                                     '{}_expected_results'.format(platform),
                                     'multifit.txt')

        actual_file = os.path.join(os.path.dirname(__file__),
                                   'fitbenchmarking_results',
                                   'multifit_set',
                                   'multifit_set_acc_weighted_table.txt')

        with open(expected_file, 'r') as f:
            expected = f.readlines()

        with open(actual_file, 'r') as f:
            actual = f.readlines()

        diff, msg = diff_result(actual, expected)
        self.assertListEqual([], diff, msg)


@pytest.mark.skipif("TEST_TYPE == 'all'")
class TestRegressionDefault(TestCase):
    """
    Regression tests for the Fitbenchmarking software with all default fitting
    software packages
    """

    @classmethod
    def setUpClass(cls):
        """
        Create an options file, run it, and get the results.
        """

        # Get defaults which should have minimizers for every software
        opts = setup_options()
        opt_file = tempfile.NamedTemporaryFile(suffix='.ini',
                                               mode='w',
                                               delete=False)
        # while opt_file is open it cannot be reoponed on Windows NT or later
        # and hence option available is to write to the stream directly
        opts.write_to_stream(opt_file)
        opt_file.close()
        problem = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                               os.pardir,
                                               'mock_problems',
                                               'default_parsers'))

        run([problem], options_file=opt_file.name, debug=True)
        os.remove(opt_file.name)

    def test_results_consistent(self):
        """
        Regression testing that the results of fitting a set of problems
        containing all problem types against a single minimizer from each of
        the supported softwares
        """

        expected_file = os.path.join(os.path.dirname(__file__),
                                     '{}_expected_results'.format(platform),
                                     'default_parsers.txt')

        actual_file = os.path.join(os.path.dirname(__file__),
                                   'fitbenchmarking_results',
                                   'default_parsers',
                                   'default_parsers_acc_weighted_table.txt')

        with open(expected_file, 'r') as f:
            expected = f.readlines()

        with open(actual_file, 'r') as f:
            actual = f.readlines()

        diff, msg = diff_result(actual, expected)
        self.assertListEqual([], diff, msg)


def diff_result(expected, actual):
    """
    Return the lines which differ between expected and actual along with a
    formatted message.

    :param expected: The expected result
    :type expected: list of strings
    :param actual: The actual result
    :type actual: list of strings
    :return: The lines which differ and a formatted message
    :rtype: list of list of strings and str
    """
    diff = []
    for exp_line, act_line in zip_longest(expected, actual):
        exp_line = '' if exp_line is None else exp_line.strip('\n')
        act_line = '' if act_line is None else act_line.strip('\n')
        if exp_line != act_line:
            diff.append([exp_line, act_line])

    num_diff = min(6, len(diff))
    msg = 'Accuracy has changed in at least 1 minimizer-' \
          + 'problem pair. \n' \
          + 'First {} of {} differences: \n'.format(num_diff, len(diff)) \
          + '\n'.join(['{} \n{}'.format(*diff[i])
                       for i in range(num_diff)])
    return diff, msg


def setup_options(multifit=False):
    """
    Setups up options class for system tests

    :return: Fitbenchmarking options file for tests
    :rtype: fitbenchmarking.utils.options.Options
    """

    # Get defaults which should have minimizers for every software
    opts = Options()
    opts.num_runs = 1
    opts.make_plots = False
    # Use only the first minimizer from the selected software packages
    if multifit:
        opts.software = ['mantid']
        opts.minimizers = {'mantid': [opts.minimizers['mantid'][0]]}
    elif TEST_TYPE != "default":
        opts.software = ['bumps', 'dfo', 'gsl', 'mantid', 'minuit',
                         'ralfit', 'scipy', 'scipy_ls']
        opts.minimizers = {k: [v[0]] for k, v in opts.minimizers.items()}
    else:
        opts.software = ['bumps', 'dfo', 'minuit', 'scipy', 'scipy_ls']
        opts.minimizers = {s: [opts.minimizers[s][0]] for s in opts.software}

    opts.results_dir = os.path.join(os.path.dirname(__file__),
                                    'fitbenchmarking_results')

    return opts



from __future__ import absolute_import, division, print_function

import time
import os
import shutil
import unittest

from fitbenchmarking.utils.create_dirs import (figures, group_results, results,
                                               support_pages, css)


class CreateDirsTests(unittest.TestCase):

    def setUp(self):
        path = 'r{}'.format(int(time.time()))
        self.results_dir = os.path.join(os.getcwd(), path)

    def tearDown(self):
        if os.path.exists(self.results_dir):
            shutil.rmtree(self.results_dir)

    def test_results_throw_correct_error(self):

        self.assertRaises(TypeError, results, 123)
        self.assertRaises(TypeError, results, None)

    def test_results_create_correct_dir(self):

        results_dir = results(self.results_dir)
        results_dir_expected = self.results_dir

        self.assertEqual(results_dir_expected, results_dir)
        self.assertTrue(os.path.exists(results_dir_expected))

        shutil.rmtree(results_dir_expected)

    def test_groupResults_create_correct_group_results(self):

        results_dir = results(self.results_dir)
        group_results_dir = group_results(results_dir, "test_group")
        group_results_dir_expected = os.path.join(results_dir, "test_group")

        self.assertEqual(group_results_dir_expected, group_results_dir)
        self.assertTrue(os.path.exists(group_results_dir_expected))

        shutil.rmtree(results_dir)

    def test_support_pages_create_correct_dir(self):

        results_dir = results(self.results_dir)
        group_results_dir = group_results(results_dir, "test_group")
        support_pages_dir = support_pages(group_results_dir)
        support_pages_dir_expected = os.path.join(group_results_dir,
                                                  'support_pages')

        self.assertEqual(support_pages_dir_expected, support_pages_dir)
        self.assertTrue(os.path.exists(support_pages_dir_expected))

        shutil.rmtree(results_dir)

    def test_figures_create_correct_dir(self):

        results_dir = results(self.results_dir)
        group_results_dir = group_results(results_dir, "test_group")
        support_pages_dir = support_pages(group_results_dir)

        figures_dir = figures(support_pages_dir)
        figures_dir_expected = os.path.join(support_pages_dir, 'figures')

        self.assertEqual(figures_dir_expected, figures_dir)
        self.assertTrue(os.path.exists(figures_dir_expected))

        shutil.rmtree(results_dir)

    def test_css_create_correct_dir(self):

        results_dir = results(self.results_dir)
        group_results_dir = group_results(results_dir, "test_group")
        css_dir = css(group_results_dir)
        css_dir_expected = os.path.join(group_results_dir,
                                    'css')
        
        self.assertEqual(css_dir_expected, css_dir)
        self.assertTrue(os.path.exists(css_dir_expected))

        shutil.rmtree(css_dir)


if __name__ == "__main__":
    unittest.main()
